// const path = require('path');
const fs = require('fs-extra');

const extend = require('extend');
// const Q = require('q');
// const _ = require('underscore');
// const moment = require('moment');
const babel = require(`@babel/core`);

/**
 * NL类库
 *
 * @module nllib
 */

/**
 * 通过babel，转换es6代码到es5代码
 *
 * @class nllib/src/babel/transform
 * @static
 *
 * @param {object} pParameter
 *
 * @param {string} pParameter.type 类型，000|100
 * <br>000:要求传入代码字符串
 * <br>100:要求传入文件绝对路径
 * @param {object} pParameter.typeParameter[`000`].code 代码字符串，仅type=000时有效
 * @param {object} pParameter.typeParameter[`100`].path_file 代码文件路径，仅type=100时有效
 *
 * @param {boolean} pParameter.log_babel_transform_options 是否输出传入到babel.transform的options，默认值为false
 * @param {object} pParameter.babel_transform_options 传入到babel.transform的options，默认值为
 * <br >{
 * <br >babelrc: false,
 * <br >comments: false,
 * <br >configFile: false,
 * <br >minified: true,
 * <br >presets: [`@babel/preset-env`],
 * <br >}
 * <br >参考：<a target="__blank" href="https://www.babeljs.cn/docs/options">https://www.babeljs.cn/docs/options</a>
 * @return {object} 转换结果
 *
 * @demo src/babel/transform/1.0.1/demo/index.demo.js
 */

const transform = function (pParameter) {
    const defaultParameter = {};
    defaultParameter.log_babel_transform_options = false;
    defaultParameter.babel_transform_options = {
        // filename: `filename`,
        babelrc: false,
        comments: false,
        configFile: false,

        minified: true,

        presets: [
            `@babel/preset-env`,
        ],
    };

    defaultParameter.type = '000';

    defaultParameter.typeParameter = {};

    defaultParameter.typeParameter[`000`] = {};
    defaultParameter.typeParameter[`000`].code = ``;

    defaultParameter.typeParameter[`100`] = {};
    defaultParameter.typeParameter[`100`].path_file = ``;

    const parameter = extend(true, {}, defaultParameter, pParameter);
    const type = parameter.type;

    const options = parameter.babel_transform_options;
    if (parameter.log_babel_transform_options) {
        console.log(JSON.stringify(options, null, 4));
    }

    const process_type_000 = function () {
        const code = parameter.typeParameter[`000`].code;

        return babel.transformSync(code, options);
    };

    const process_type_100 = function () {
        const path_file = parameter.typeParameter[`100`].path_file;
        const code = fs.readFileSync(path_file, `utf-8`);

        return babel.transformSync(code, options);
    };

    if (type === `000`) return process_type_000();
    if (type === `100`) return process_type_100();
};

module.exports = transform;

//参考
//https://babeljs.io/docs/en/options#filename
