const extend = require(`extend`);

const to_object = require(`../json/to_object`);

/**
 * NL类库
 *
 * @module NL
 */

/**
 * 缓存操作封装类
 * <div>
 * <span class="tag margin-right">支持浏览器环境</span>
 * <span class="tag margin-right">不支持node环境</span>
 * </div>
 *
 * @class NL.cache
 * @static
 *
 */

/**
 * 获取缓存值
 *
 * @method get
 *
 * @param {object} pParameter
 * @param {string} pParameter.key 缓存key
 *
 * @return {object} 缓存value
 *
 * @demo src/cache/demo/get.demo.js
 */

const get = function (pParameter) {
    if (typeof pParameter !== `object` && arguments.length === 1) {
        pParameter = {
            key: arguments[0]
        };
    }

    const defaultParameter = {};
    defaultParameter.key = '';
    defaultParameter.type = `LocalStorage`;//LocalStorage||Cookie

    const parameter = extend(true, {}, defaultParameter, pParameter);

    const key = parameter.key;

    let json = null;
    switch (parameter.type) {
        case `LocalStorage`:
            json = global.localStorage.getItem(key);
            break;

        case `Cookie`:
            console.error(`还未实现！`);
            break;
    }
    const value = to_object(json);
    return value;
};

module.exports = get;
