const extend = require(`extend`);

const to_json = require(`../json/to_json`);

/**
 * NL类库
 *
 * @module NL
 */

/**
 * 缓存操作封装类
 * <div>
 * <span class="tag margin-right">支持浏览器环境</span>
 * <span class="tag margin-right">不支持node环境</span>
 * </div>
 *
 * @class NL.cache
 * @static
 *
 */

/**
 * 设置缓存值
 *
 * @method set
 *
 * @param {object} pParameter
 * @param {string} pParameter.key 缓存key
 * @param {object} pParameter.value 缓存value
 *
 * @demo src/cache/demo/set.demo.js
 */

const set = function (pParameter) {
    if (typeof pParameter !== `object` && arguments.length === 2) {
        pParameter = {
            key: arguments[0],
            value: arguments[1]
        };
    }

    const defaultParameter = {};
    defaultParameter.key = ``;
    defaultParameter.value = ``;
    defaultParameter.type = `LocalStorage`;//LocalStorage||Cookie

    const parameter = extend(true, {}, defaultParameter, pParameter);

    const key = parameter.key;
    const value = parameter.value;

    const json = to_json(value);
    switch (parameter.type) {
        case `LocalStorage`:
            global.localStorage.setItem(key, json);
            break;

        case `Cookie`:
            console.error(`还未实现！`);
            break;
    }
};

module.exports = set;
