const Q = require('q');
const _ = require('underscore');
const to_boolean = require('../../../convert/to_boolean');
const exec = require('../..//exec');

const exec_cmd = function (pParameter) {
    if (!pParameter) pParameter = {};
    const command = pParameter.command;
    const options = pParameter.options;
    const stdout_data_event_handler = pParameter.stdout_data_event_handler;
    const stderr_data_event_handler = pParameter.stderr_data_event_handler;
    const exit_event_handler = pParameter.exit_event_handler;
    const close_event_handler = pParameter.close_event_handler;
    const enable_console = to_boolean(pParameter.enable_console, true);

    if (!command) {
        const msg = 'exec_cmd parameter error, code=001';
        console.error(msg);
        console.trace();
        return Q.reject({msg: msg});
    }

    const defer_exit = Q.defer();
    const defer_close = Q.defer();

    const returnValueExec = exec(command, options);
    returnValueExec.stdout.on(`data`, function (chunk) {
        if (_.isFunction(stdout_data_event_handler)) {
            stdout_data_event_handler(chunk);
            return;
        }
        const textChunk = chunk.toString(`utf8`);
        if (enable_console) console.log(textChunk);
    });
    returnValueExec.stderr.on(`data`, function (chunk) {
        if (_.isFunction(stderr_data_event_handler)) {
            stderr_data_event_handler(chunk);
            return;
        }
        const textChunk = chunk.toString(`utf8`);
        if (enable_console) console.log(textChunk);
    });

    returnValueExec.on(`exit`, function (code) {
        if (_.isFunction(exit_event_handler)) {
            exit_event_handler(code);
            if (code === 0) {
                defer_exit.resolve({code});
            } else {
                defer_exit.reject({code});
            }
            return;
        }
        if (enable_console) console.log(`exit code : ${code}`);
        if (code === 0) {
            defer_exit.resolve({code});
        } else {
            defer_exit.reject({code});
        }
    });
    returnValueExec.on(`close`, function (code) {
        if (_.isFunction(close_event_handler)) {
            close_event_handler(code);
            defer_close.resolve();
            return;
        }
        if (enable_console) console.log(`close code : ${code}`);
        defer_close.resolve();
    });

    return Q.all([
        defer_exit.promise,
        defer_close.promise,
    ]);
};
module.exports = exec_cmd;

// 参考：
// https://stackoverflow.com/questions/37522010/difference-between-childprocess-close-exit-events
// https://stackoverflow.com/questions/18894433/nodejs-child-process-working-directory
