//<editor-fold desc="node core modules">
const path = require('path');
//</editor-fold>

//<editor-fold desc="npm modules">
const fs = require('fs-extra');
const extend = require('extend');
const Q = require('q');
const _ = require('underscore');
const moment = require('moment');
const URI = require('urijs');
const filesize = require('filesize');
const got = require('got');
//</editor-fold>

const download_file_by_url = function (pParameter) {
    const defaultParameter = {};
    defaultParameter.url_src = '';
    defaultParameter.path_dest = '';
    defaultParameter.download_progress_display_type = 'multi-line';//multi-line|single-line

    const parameter = extend(true, {}, defaultParameter, pParameter);

    const url_src = parameter.url_src;
    const path_dest = parameter.path_dest;
    const download_progress_display_type = parameter.download_progress_display_type;

    const path_dirname_dest = path.dirname(path_dest);
    fs.ensureDirSync(path_dirname_dest);

    const writeStream = fs.createWriteStream(path_dest);

    const defer = Q.defer();

    let percent_last = 0;
    console.log(`文件下载开始：${url_src}`);
    const readStream = got.stream(url_src);
    readStream
        .on('downloadProgress', function (progress) {
            // console.log(progress);

            const percent_001 = progress.percent;
            const percent_002 = percent_001 * 100;
            const percent_003 = percent_002.toFixed(2);
            const percent_004 = `${percent_003}%`;

            const transferred_001 = progress.transferred;
            const transferred_002 = filesize(transferred_001);

            const total_001 = progress.total;
            const total_002 = filesize(total_001);

            let msg = ``;
            msg += `下载进度：${percent_004}；`;
            msg += `已经下载大小：${transferred_002}；`;
            msg += `需要下载大小：${total_002}；`;

            const delta = percent_001 - percent_last;
            // console.log(`delta=${delta}`);
            if (delta >= 0.01) {
                switch (download_progress_display_type) {
                    case `single-line`:
                        process.stdout.write(msg);
                        break;

                    case `multi-line`:
                    default:
                        console.log(msg);
                        break;
                }

                percent_last = progress.percent;
            }
        })
        .on('end', function () {
            defer.resolve();
            console.log();
            console.log(`文件下载结束：${url_src} ===> ${path_dest}`);
        })
        .on(`error`, function (ex) {
            console.error(`文件下载异常：${url_src} ===> ${path_dest}`);
            console.error(ex);
            if (ex) {
                if (ex.stack) {
                    console.error(ex.stack);
                }
            }
            defer.reject(ex);
        })
    ;
    readStream.pipe(writeStream);

    return defer.promise;
};

module.exports = download_file_by_url;
