//<editor-fold desc="node core modules">
const path = require('path');
//</editor-fold>

//<editor-fold desc="npm modules">
const fs = require('fs-extra');
const extend = require('extend');
const Q = require('q');
const _ = require('underscore');
const moment = require('moment');
const URI = require('urijs');
const filesize = require('filesize');
const got = require('got');
//</editor-fold>

/**
 * NL类库
 *
 * @module nllib
 */


/**
 * 下载远程文件
 *
 * @class nllib/src/common/download_file_by_url
 * @static
 *
 * @param {object} pParameter
 * @param {string} pParameter.url_src 远程文件url
 * @param {string} pParameter.path_dest 本地文件路径
 * @param {string} pParameter.download_progress_display_type 下载进度显示类型（multi-line|single-line），默认为multi-line
 * @param {boolean} pParameter.enable_log 是否输出日志，默认为true
 * @return {object} 远程文件下载完成的promise
 *
 * @demo src/common/download_file_by_url/1.0.1/demo/index.demo.js
 */

const download_file_by_url = function (pParameter) {
    const defaultParameter = {};
    defaultParameter.url_src = '';
    defaultParameter.path_dest = '';
    defaultParameter.download_progress_display_type = 'multi-line';//multi-line|single-line
    defaultParameter.enable_log = true;

    const parameter = extend({}, defaultParameter, pParameter);

    const url_src = parameter.url_src;
    const path_dest = parameter.path_dest;
    const download_progress_display_type = parameter.download_progress_display_type;
    const enable_log = parameter.enable_log;

    const path_dirname_dest = path.dirname(path_dest);
    fs.ensureDirSync(path_dirname_dest);

    const writeStream = fs.createWriteStream(path_dest);

    const defer = Q.defer();

    let percent_last = 0;
    if (enable_log) console.log(`文件下载开始：${url_src}`);
    const readStream = got.stream(url_src);
    readStream
        .on('downloadProgress', function (progress) {
            // console.log(progress);

            const percent_001 = progress.percent;
            const percent_002 = percent_001 * 100;
            const percent_003 = percent_002.toFixed(2);
            const percent_004 = `${percent_003}%`;

            const transferred_001 = progress.transferred;
            const transferred_002 = filesize(transferred_001);

            const total_001 = progress.total;
            const total_002 = filesize(total_001);

            let msg = ``;
            msg += `下载进度：${percent_004}；`;
            msg += `已经下载大小：${transferred_002}；`;
            msg += `需要下载大小：${total_002}；`;

            const delta = percent_001 - percent_last;
            // console.log(`delta=${delta}`);
            if (delta >= 0.01) {
                switch (download_progress_display_type) {
                    case `single-line`:
                        if (enable_log) process.stdout.write(msg);
                        break;

                    case `multi-line`:
                    default:
                        if (enable_log) console.log(msg);
                        break;
                }

                percent_last = progress.percent;
            }
        })
        .on('end', function () {
            defer.resolve();
            if (enable_log) console.log();
            if (enable_log) console.log(`文件下载结束：${url_src} ===> ${path_dest}`);
        })
        .on(`error`, function (ex) {
            console.error(`文件下载异常：${url_src} ===> ${path_dest}`);
            console.error(ex);
            if (ex) {
                if (ex.stack) {
                    console.error(ex.stack);
                }
            }
            defer.reject(ex);
        })
    ;
    readStream.pipe(writeStream);

    return defer.promise;
};

module.exports = download_file_by_url;
