//<editor-fold desc="node core modules">
const path = require('path');
const crypto = require('crypto');
//</editor-fold>

//<editor-fold desc="npm modules">
const fs = require('fs-extra');
const extend = require('extend');
const Q = require('q');
const _ = require('underscore');
const moment = require('moment');
const URI = require('urijs');
const rp = require('request-promise');
//</editor-fold>

/**
 * NL类库
 *
 * @module NL
 */


/**
 * NL.common
 *
 * @class NL.common
 * @static
 *
 */

/**
 * 根据指定参数生成md5字符串
 * <div>
 * <span class="tag margin-right">不支持浏览器环境</span>
 * <span class="tag margin-right">支持node环境</span>
 * </div>
 *
 * @method get_md5
 * @param {object} pParameter
 * @param {string} pParameter.type 类型（000|100|200）,默认为000
 * @param {string} pParameter.typeParameter['000'].path type=000时有效，本地文件路径
 * @param {string} pParameter.typeParameter['100'].buffer type=100时有效，buffer或者字符串
 * @param {string} pParameter.typeParameter['200'].uri type=200时有效，远程地址
 * @param {string} pParameter.typeParameter['200'].options type=200时有效，发起请求的相关参数，详情参考：https://github.com/request/request
 * @return {object} promise
 * <br>promise.then((md5) => {console.log(md5);});
 *
 * @demo src/common/demo/get_md5.demo.js
 */
const get_md5 = function (pParameter) {
    if (!pParameter) {
        pParameter = {};
    }

    const defaultParameter = {};
    defaultParameter.type = `000`;//000||100
    defaultParameter.typeParameter = {};
    defaultParameter.typeParameter[`000`] = {};
    defaultParameter.typeParameter[`000`].path = ``;

    defaultParameter.typeParameter[`100`] = {};
    defaultParameter.typeParameter[`100`].buffer = null;

    defaultParameter.typeParameter[`200`] = {};
    defaultParameter.typeParameter[`200`].uri = ``;
    defaultParameter.typeParameter[`200`].options = {};

    const parameter = extend(true, {}, defaultParameter, pParameter);

    const type = parameter.type;

    // console.log(`pParameter.type=${pParameter.type}`);
    // console.log(`type=${type}`);

    const process_type_000 = function () {
        const __path = parameter.typeParameter[`000`].path;

        const buffer = fs.readFileSync(__path);
        const hash = crypto.createHash('md5');
        hash.update(buffer);
        const value = hash.digest('hex');

        return Q.resolve(value);
    };

    const process_type_100 = function () {
        const buffer = parameter.typeParameter[`100`].buffer;

        const hash = crypto.createHash('md5');
        hash.update(buffer);
        const value = hash.digest('hex');
        return Q.resolve(value);
    };

    const process_type_200 = function () {
        const uri = parameter.typeParameter[`200`].uri;
        const __options = parameter.typeParameter[`200`].options;

        const options = extend(true, {}, __options);
        options.encoding = null;
        options.uri = uri;

        return Q.when()
            .then(function () {
                return rp(options);
            })
            .then(function (buffer) {
                const hash = crypto.createHash('md5');
                hash.update(buffer);
                const value = hash.digest('hex');
                return Q.resolve(value);
            })
            .catch(function (ex) {
                console.error(ex);
                if (ex) {
                    if (ex.stack) {
                        console.error(ex.stack);
                    }
                }
                return Q.reject(ex);
            })
            ;
    };

    let r;
    switch (type) {
        case `000`:
            r = process_type_000();
            break;

        case `100`:
            r = process_type_100();
            break;

        case `200`:
            r = process_type_200();
            break;
    }

    return r;
};

module.exports = get_md5;
