const extend = require('extend');
const _ = require('underscore');

const process = global.process;

/**
 * NL类库
 *
 * @module nllib
 */


/**
 * 通过命令行获取参数
 *
 * @class nllib/src/common/get_parameter_from_cli
 * @static
 * @param {object} pParameter
 * @param {string} pParameter.type 类型，000|100|200，默认值为200
 * <br />000:返回的对象，为原始对象
 * <br />100:返回的对象，属性名全部变为小写
 * <br />200:返回的对象，属性名全部变为小写，去掉最外层的配对的单引号，或者配对的双引号
 * @param {array} pParameter.argv argv，默认值为process.argv
 * @return {object} 参数对象
 *
 * @demo src/common/demo/get_parameter_from_cli.demo.js
 */
const get_parameter_from_cli = function (pParameter) {
    if (!pParameter) {
        pParameter = {};
    }

    const defaultParameter = {};
    defaultParameter.type = '200';
    defaultParameter.argv = process.argv;

    const parameter = extend({}, defaultParameter, pParameter);

    const type = parameter.type;
    const argv = parameter.argv;
    // console.log(JSON.stringify(defaultParameter.argv, null, 4));
    // console.log(JSON.stringify(argv, null, 4));

    const process_type_000 = function () {
        const get_p = function () {
            const reg = /--(.*)=(.*)/gi;

            const p = {};

            if (argv.length < 3) {
                return p;
            }

            for (let i = 2; i < argv.length; i++) {
                const s = argv[i];
                let result;
                while ((result = reg.exec(s)) !== null) {
                    // console.log(result);
                    const property_name = result[1];
                    const property_value = result[2];
                    p[property_name] = property_value;
                }
            }

            return p;
        };
        const p = get_p();
        return p;
    };

    const process_type_100 = function () {
        const get_p = function () {
            const reg = /--(.*)=(.*)/gi;

            const p = {};

            if (argv.length < 3) {
                return p;
            }

            for (let i = 2; i < argv.length; i++) {
                const s = argv[i];
                let result;
                while ((result = reg.exec(s)) !== null) {
                    // console.log(result);
                    const property_name = result[1];
                    const property_value = result[2];
                    p[property_name] = property_value;
                }
            }

            return p;
        };
        const p = get_p();

        const o = {};
        _.each(p, function (property_value, property_name) {
            const name = property_name.toLowerCase();
            o[name] = property_value;
        });
        return o;
    };

    const process_type_200 = function () {
        const reg_001 = /--(.*)=(.*)/gi;
        const reg_002 = /'(.*)'/gi;
        const reg_003 = /"(.*)"/gi;

        const get_p = function () {
            const p = {};

            if (argv.length < 3) {
                return p;
            }

            for (let i = 2; i < argv.length; i++) {
                const s = argv[i];
                let result;
                while ((result = reg_001.exec(s)) !== null) {
                    // console.log(result);
                    const property_name = result[1];
                    const property_value_001 = result[2];
                    let property_value_002 = property_value_001;
                    if (property_value_001.match(reg_002)) {
                        property_value_002 = property_value_001.replace(reg_002, `$1`);
                    } else if (property_value_001.match(reg_003)) {
                        property_value_002 = property_value_001.replace(reg_003, `$1`);
                    }
                    p[property_name] = property_value_002;
                }
            }

            return p;
        };
        const p = get_p();

        const o = {};
        _.each(p, function (property_value, property_name) {
            const name = property_name.toLowerCase();
            o[name] = property_value;
        });
        return o;
    };

    if (type === `000`) return process_type_000();
    if (type === `100`) return process_type_100();
    if (type === `200`) return process_type_200();
};

module.exports = get_parameter_from_cli;
