//<editor-fold desc="node core modules">
const path = require('path');
//</editor-fold>

//<editor-fold desc="npm modules">
const fs = require('fs-extra');
const extend = require('extend');
const Q = require('q');
const _ = require('underscore');
const moment = require('moment');
const URI = require('urijs');
//</editor-fold>

const to_boolean = require(`../convert/to_boolean`);
const to_int = require(`../convert/to_int`);

/**
 * NL类库
 *
 * @module nllib
 */

/**
 * 根据express的req对象获取参数值
 *
 * @class nllib/src/common/get_parameter_value_by_req
 * @static
 *
 * @param {object} pParameter
 * @param {object} pParameter.req req对象
 * @param {string} pParameter.property_name 属性名称
 * @param {object} pParameter.data_type 数据类型（string|boolean|int），默认值为string
 * @param {object} pParameter.default_value 获取失败时的默认值
 * @param {boolean} pParameter.is_trim 是否执行trim操作，默认值为true
 * @param {boolean} pParameter.is_decodeURIComponent 是否执行decodeURIComponent操作，默认值为true
 * @return {object} 参数值
 *
 * @demo src/common/demo/get_parameter_value_by_req.demo.js
 */

const get_parameter_value_by_req = function (pParameter) {
    const defaultParameter = {};
    defaultParameter.req = undefined;
    defaultParameter.property_name = ``;
    defaultParameter.data_type = `string`;//string|boolean|int
    defaultParameter.default_value = ``;
    defaultParameter.is_trim = true;
    defaultParameter.is_decodeURIComponent = true;

    const parameter = extend(true, {}, defaultParameter, pParameter);

    const req = parameter.req;
    const property_name = parameter.property_name;
    const default_value = parameter.default_value;
    const is_trim = parameter.is_trim;
    const is_decodeURIComponent = parameter.is_decodeURIComponent;
    const data_type = parameter.data_type;

    const value_001 = (function () {
        let r;

        if (req) {
            if (req.query) {
                if (req.query[property_name]) {
                    r = req.query[property_name];
                    return r;
                }
            }

            if (req.body) {
                if (req.body[property_name]) {
                    r = req.body[property_name];
                    return r;
                }
            }
        }

        if (r === undefined) {
            if (default_value !== undefined) {
                r = default_value;
                return r;
            }
        }

    })();

    let value_002;
    if (is_decodeURIComponent) {
        value_002 = decodeURIComponent(value_001);
    } else {
        value_002 = value_001;
    }

    let value_003;
    if (is_trim) {
        value_003 = value_002.trim();
    } else {
        value_003 = value_002;
    }

    let value_004;
    switch (data_type) {
        case `string`:
            value_004 = value_003;
            break;

        case `boolean`:
        case `bool`:
            value_004 = to_boolean(value_003, default_value);
            break;

        case `int`:
            value_004 = to_int(value_003, default_value);
            break;

        default:
            value_004 = value_003;
            break;
    }

    return value_004;
};

module.exports = get_parameter_value_by_req;
