const _ = require(`underscore`);

/**
 * NL类库
 *
 * @module nllib
 */

/**
 * 根据传入的实体列表，过滤出指定数量的、互不重复的、随机的、实体列表，并返回
 *
 * @class nllib/src/common/get_random_entity_list
 * @static
 *
 * @param {object} pParameter
 * @param {array} pParameter.entity_list 实体列表，必填
 * @param {int} pParameter.count 返回的实体数量，必填
 * @param {function} pParameter.get_entity_unique_id 获取实体的唯一id，方法默认值为
 <div>
 const __default_get_entity_unique_id = function (p) {
        if (!p) p = {};
        const entity = p.entity;
        return JSON.stringify(entity);
    };
 * @return {array} 指定数量的、互不重复的、随机的、实体列表
 *
 * @demo src/common/get_random_entity_list/1.0.0/demo/index.demo.js
 */
const get_random_entity_list = function (pParameter) {
    if (!pParameter) pParameter = {};
    const entity_list = pParameter.entity_list;
    const count = pParameter.count;
    const __get_entity_unique_id = pParameter.get_entity_unique_id;

    const __default_get_entity_unique_id = function (p) {
        if (!p) p = {};
        const entity = p.entity;
        return JSON.stringify(entity);
    };

    if (!_.isArray(entity_list)) {
        const msg = `get_random_entity_list parameter error, code=001`;
        console.error(msg);
        console.trace();
        return [];
    }

    if (count <= 0) {
        const msg = `get_random_entity_list parameter error, code=002`;
        console.error(msg);
        console.trace();
        return [];
    }

    if (count > entity_list.length) {
        const msg = `get_random_entity_list parameter error, code=003`;
        console.error(msg);
        console.trace();
        return [];
    }

    let get_entity_unique_id;
    if (_.isFunction(__get_entity_unique_id)) {
        get_entity_unique_id = __get_entity_unique_id;
    } else {
        get_entity_unique_id = __default_get_entity_unique_id;
    }

    const entity_list_001 = _.chain(entity_list)
        .map(function (entity) {
            const unique_id = get_entity_unique_id({
                entity,
            });
            return {
                unique_id,
                entity,
            }
        })
        .value()
    ;

    const result_entity_list_001 = [];

    let n = count - 1;
    while (n >= 0) {

        const unique_id_list = _.chain(result_entity_list_001)
            .map(function (m) {
                const unique_id = m.unique_id;
                return unique_id;
            })
            .value()
        ;

        const list = _.chain(entity_list_001)
            .filter(function (m) {
                const unique_id = m.unique_id;
                return !_.contains(unique_id_list, unique_id);
            })
            .value()
        ;

        const index = _.random(0, list.length - 1);
        const unique_id = list[index].unique_id;
        const entity = list[index].entity;

        result_entity_list_001.push({
            unique_id,
            entity,
        });

        n--;
    }

    const result_entity_list_002 = _.chain(result_entity_list_001)
        .map(function (m) {
            const entity = m.entity;
            return entity;
        })
        .value()
    ;
    return result_entity_list_002;
};

module.exports = get_random_entity_list;
