const extend = require(`extend`);
const _ = require(`underscore`);

/**
 * NL类库
 *
 * @module nllib
 */

/**
 * 字典封装类
 *
 * @class nllib/src/dict
 * @constructor
 *
 * @param {object} pInitParameter
 * @param {string} pInitParameter.init_type 初始化类型（000），默认值为000
 * @param {string} pInitParameter.init_parameter.000.entity_define 实体定义，仅init_type=000时有效
 * @param {string} pInitParameter.init_parameter.000.entity_define.property_name_for_value 字典项value的属性名称
 * @param {string} pInitParameter.init_parameter.000.entity_define.property_name_for_text 用于字典项text的属性名称
 * @param {string} pInitParameter.init_parameter.000.entity_list 实体列表，仅init_type=000时有效
 * <br>entity_list中的实体，必须具有entity_define中property_name_for_value和property_name_for_text对应的属性
 *
 * @return {object} nllib/src/dict实例
 *
 * @demo src/dict/1.0.1/demo/index.demo.js
 *
 */
const Dict = function Dict(pInitParameter) {
    const __this = this;

    const default_init_parameter = {};

    default_init_parameter.init_type = `000`;
    default_init_parameter.init_parameter = {};
    default_init_parameter.init_parameter[`000`] = {};
    default_init_parameter.init_parameter[`000`].entity_define = {
        property_name_for_text: `text`,
        property_name_for_value: `value`,
    };
    default_init_parameter.init_parameter[`000`].entity_list = null;

    const __init_parameter = extend({}, default_init_parameter, pInitParameter);

    //<editor-fold desc="兼容旧字段">
    if (pInitParameter) {
        if (pInitParameter.initType) {
            __init_parameter.init_type = pInitParameter.initType;
        }

        if (pInitParameter.initParameter) {
            if (pInitParameter.initParameter[`000`]) {
                if (pInitParameter.initParameter[`000`].entityDefine) {
                    if (pInitParameter.initParameter[`000`].entityDefine.text) {
                        __init_parameter.init_parameter[`000`].entity_define.property_name_for_text = pInitParameter.initParameter[`000`].entityDefine.text;
                    }

                    if (pInitParameter.initParameter[`000`].entityDefine.value) {
                        __init_parameter.init_parameter[`000`].entity_define.property_name_for_value = pInitParameter.initParameter[`000`].entityDefine.value;
                    }

                    if (_.isArray(pInitParameter.initParameter[`000`].entityList)) {
                        __init_parameter.init_parameter[`000`].entity_list = pInitParameter.initParameter[`000`].entityList;
                    }
                }
            }
        }
    }
    //</editor-fold>

    const init_type = __init_parameter.init_type;
    const init_parameter = __init_parameter.init_parameter;

    const process_init_type_000 = function () {
        const init_parameter_000 = init_parameter[`000`];
        const entity_define = init_parameter_000.entity_define;
        const entity_list = init_parameter_000.entity_list;

        const property_name_for_value = entity_define.property_name_for_value;
        const property_name_for_text = entity_define.property_name_for_text;

        const list_001 = _.map(entity_list, function (m) {
            return m[property_name_for_text];
        });
        const list_002 = _.map(entity_list, function (m) {
            return m[property_name_for_value];
        });

        const text_value_map = _.object(list_001, list_002);
        const value_text_map = _.object(list_002, list_001);

        const text_value_map_entity = _.chain(text_value_map)
            .mapObject(function (property_value) {
                const entity = _.find(entity_list, function (m) {
                    return m[property_name_for_value] === property_value;
                });
                return entity;
            })
            .value();

        const value_text_map_entity = _.chain(value_text_map)
            .mapObject(function (property_value) {
                const entity = _.find(entity_list, function (m) {
                    return m[property_name_for_text] === property_value;
                });
                return entity;
            })
            .value();

        //<editor-fold desc="不建议使用的属性">
        __this.entityList = entity_list;
        __this.map = text_value_map;
        __this._map = value_text_map;
        __this.mapEntity = text_value_map_entity;
        __this._mapEntity = value_text_map_entity;
        //</editor-fold>

        const get_entity_list = function () {
            return entity_list;
        };

        const get_text_value_map = function () {
            return text_value_map;
        };

        const get_value_text_map = function () {
            return value_text_map;
        };

        const get_text_entity_map = function () {
            return text_value_map_entity;
        };

        const get_value_entity_map = function () {
            return value_text_map_entity;
        };

        const get_entity_by_value = function (pValue) {
            const entity = _.chain(entity_list)
                .find(function (m) {
                    const value = m[property_name_for_value];
                    return pValue === value;
                })
                .value()
            ;
            return entity;
        };

        const get_text_by_value = function (pValue) {
            const entity = get_entity_by_value(pValue);
            let text = ``;
            if (entity) {
                text = entity[property_name_for_text];
            }
            return text;
        };

        const get_entity_by_text = function (pText) {
            const entity = _.chain(entity_list)
                .find(function (m) {
                    const text = m[property_name_for_text];
                    return pText === text;
                })
                .value()
            ;
            return entity;
        };

        const get_value_by_text = function (pText) {
            const entity = get_entity_by_text(pText);
            let value = ``;
            if (entity) {
                value = entity[property_name_for_value];
            }
            return value;
        };

        /**
         * 获取实体列表
         *
         * @method get_entity_list
         * @return {array} 实体列表
         */
        __this.get_entity_list = get_entity_list;

        /**
         * 获取text、value映射对象
         *
         * @method get_text_value_map
         * @return {object} text、value映射对象
         */
        __this.get_text_value_map = get_text_value_map;

        /**
         * 获取value、text映射对象
         *
         * @method get_value_text_map
         * @return {object} value、text映射对象
         */
        __this.get_value_text_map = get_value_text_map;

        /**
         * 获取text、entity映射对象
         *
         * @method get_text_entity_map
         * @return {object} text、entity映射对象
         */
        __this.get_text_entity_map = get_text_entity_map;

        /**
         * 获取value、entity映射对象
         *
         * @method get_value_entity_map
         * @return {object} value、entity映射对象
         */
        __this.get_value_entity_map = get_value_entity_map;

        /**
         * 根据value获取实体
         *
         * @method get_entity_by_value
         * @param {object} pValue value
         * @return {object} 实体
         */
        __this.get_entity_by_value = get_entity_by_value;

        /**
         * 根据value获取text
         *
         * @method get_text_by_value
         * @param {object} pValue value
         * @return {string} text
         */
        __this.get_text_by_value = get_text_by_value;

        /**
         * 根据text获取实体
         *
         * @method get_entity_by_text
         * @param {string} pText text
         * @return {object} 实体
         */
        __this.get_entity_by_text = get_entity_by_text;

        /**
         * 根据text获取value
         *
         * @method get_value_by_text
         * @param {string} pText text
         * @return {object} value
         */
        __this.get_value_by_text = get_value_by_text;
    };

    if (init_type === `000`) return process_init_type_000();
};

module.exports = Dict;
