const path = require(`path`);
const fs = require(`fs-extra`);

const extend = require(`extend`);
const Q = require(`q`);
const _ = require(`underscore`);
const moment = require(`moment`);

const XLSX = require('xlsx');

const to_boolean = require(`../../../convert/to_boolean`);
const init_directory = require(`../../../path/init_directory`);

/**
 * @module nllib
 */

/**
 * 根据实体列表，生成excel文件
 * <br>依赖模块：XLSX
 *
 * @class nllib/src/excel-helper/build_excel_file
 * @static
 * @param {object} pParameter
 * @param {array} pParameter.entity_list 实体列表，默认值为空数组
 * @param {string} pParameter.path_file_excel excel文件路径，默认值为当前工作目录下的{YYYY-MM-DD-HHmmss}.xls
 * @param {string} pParameter.sheetName excel工作表名称，默认值为main
 * @param {boolean} pParameter.enable_log 是否输出日志，默认值为true
 *
 * @demo src/excel-helper/build_excel_file/1.0.1/demo/index.demo.js
 */
const build_excel_file = function (pParameter) {
    if (!pParameter) pParameter = {};

    const entity_list = pParameter.entity_list || [];
    const path_file_excel = pParameter.path_file_excel || `${moment().format('YYYY-MM-DD-HHmmss')}.xls`;
    const sheetName = pParameter.sheetName || `main`;
    const enable_log = to_boolean(pParameter.enable_log, true);

    const path_dir = path.dirname(path_file_excel);
    init_directory({
        path_dir,
        enable_log,
    });

    const data_file_workbook = {
        SheetNames: [],
        Sheets: {}
    };

    const workbook = data_file_workbook;
    const query = _.find(workbook.SheetNames, function (m) {
        return m === sheetName;
    });
    if (!query) {
        workbook.SheetNames.push(sheetName);
        workbook.Sheets[sheetName] = [];
    }

    workbook.Sheets[sheetName] = XLSX.utils.json_to_sheet(entity_list);

    XLSX.writeFile(data_file_workbook, path_file_excel);
    if (enable_log) console.log(`nllib/src/excel-helper/build_excel_file:`, `${path_file_excel}`);
};

module.exports = build_excel_file;
