// const path = require(`path`);
// const fs = require(`fs-extra`);
// const extend = require(`extend`);
const Q = require(`q`);
const _ = require(`underscore`);
// const moment = require(`moment`);
const XLSX = require('xlsx');

// const to_boolean = require(`../../../convert/to_boolean`);
const base_read_excel_file = require(`../../base_read_excel_file`);

/**
 * @module nllib
 */

/**
 * 读取excel文件指定的sheet，返回数组
 * <br>依赖模块：XLSX
 *
 * @class nllib/src/excel-helper/read_excel_file
 * @static
 * @param {object} pParameter
 * @param {string} pParameter.path_file_excel excel文件路径
 * @param {string} pParameter.sheetName excel工作表名称，传入假值时，会取当前excel文件的第一个工作表名称
 * @param {string} pParameter.range 要读取数据的range，例如【D4:E999】，默认值为undefined
 *
 * @return {array} 数组
 *
 * @demo src/excel-helper/read_excel_file/1.1.1/demo/index.demo.js
 */
const read_excel_file = function (pParameter) {
    if (!pParameter) pParameter = {};
    const path_file_excel = pParameter.path_file_excel;
    const sheetName_000 = pParameter.sheetName;
    const range = pParameter.range;

    const data = base_read_excel_file({
        path_file_excel,
        range,
    });

    const aWorkBook = XLSX.readFile(path_file_excel, {});
    const sheet_data_entity_list = data.sheet_data_entity_list;
    if (sheet_data_entity_list.length <= 0) {
        const msg = `read_excel_file parameter error, code=001`;
        console.error(msg);
        console.trace();
        return Q.reject({msg: msg});
    }

    let return_entity_list;
    if (!sheetName_000) {
        return_entity_list = sheet_data_entity_list[0].entity_list;
        return return_entity_list;
    }

    const query = _.chain(sheet_data_entity_list)
        .find(function (m) {
            return m.sheet_name === sheetName_000;
        })
        .value()
    ;
    if (!query) {
        const msg = `read_excel_file parameter error, code=002`;
        console.error(msg);
        console.trace();
        return Q.reject({msg: msg});
    }

    return_entity_list = query.entity_list;
    return return_entity_list;
};

module.exports = read_excel_file;
