//<editor-fold desc="node core modules">
// const path = require('path');
//</editor-fold>

//<editor-fold desc="npm modules">
// const fs = require('fs-extra');
const extend = require('extend');
const Q = require('q');
const _ = require('underscore');
// const moment = require('moment');
// const URI = require('urijs');
const $ = require('jquery');
//</editor-fold>

const window = global;

const sourceEventEntity = {};
sourceEventEntity.eventName = '';
sourceEventEntity.handler = null;

const eventEntityList = [];

const addEventEntity = function (pParameter) {
    if (!pParameter) {
        pParameter = {};
    }

    const eventName = pParameter.eventName;
    const handler = pParameter.handler;

    const eventEntity = extend(true, {}, sourceEventEntity, {
        eventName: eventName,
        handler: handler
    });
    const query = _.find(eventEntityList, function (m) {
        return m.eventName === eventName;
    });
    if (!query) {
        eventEntityList.push(eventEntity);
    } else {
        query.handler = eventEntity.handler;
    }
};

let defer_send = null;
const postMessage = function (pParameter) {
    if (!pParameter) {
        pParameter = {};
    }

    const defaultParameter = {};
    defaultParameter.eventName = 'not-found';
    defaultParameter.data = null;
    defaultParameter.targetOrigin = '*';

    defaultParameter.type = '000';
    defaultParameter.destWindow = null;
    defaultParameter.$IFrame = null;
    defaultParameter.domIFrame = null;

    //可能是因为destWindow的关系，这里使用extend(true, {}, {});会抛出异常
    // const parameter = extend(true, {}, defaultParameter, pParameter);
    const parameter = _.extend({}, defaultParameter, pParameter);

    const getDestWindow = function () {
        let returnValue;
        switch (parameter.type) {
            case '000':
                returnValue = parameter.destWindow;
                break;

            case '001':
                returnValue = parameter.$IFrame.get(0).contentWindow;
                break;

            case '002':
                returnValue = parameter.domIFrame.contentWindow;
                break;
        }
        return returnValue;
    };

    const destWindow = getDestWindow();

    const message = {};
    message.eventName = parameter.eventName;
    message.data = parameter.data;
    destWindow.postMessage(message, parameter.targetOrigin);

    defer_send = Q.defer();
    return defer_send.promise;
};

let isInit = false;
const init = function () {
    if (isInit) {
        console.log('you seem to call NL.Common.IFrameHelper.init more than once.but NL.Common.IFrameHelper.init only effect once,even if you call it more than once.');
        return;
    }
    isInit = true;
    window.addEventListener('message', function (event) {
        const srcWindow = event.source;
        // const destWindow = event.target;

        const srcData = event.data;
        const eventName = srcData.eventName;
        if (!eventName) {
            return;
        }
        let query = _.find(eventEntityList, function (m) {
            return m.eventName === eventName;
        });
        if (!query) {
            query = {
                eventName: eventName,
                handler: function (pData) {
                    return Q.when(pData);
                }
            };
            addEventEntity(query);
        }
        const handler = query.handler;

        let returnValueHandler;
        if (_.isFunction(handler)) {
            returnValueHandler = handler({
                data: srcData.data,
                srcData: srcData,
                event: event
            });
        }

        Q.when()
            .then(function () {
                let returnValue;
                if (returnValueHandler === undefined) {
                    returnValue = Q.when();
                } else {
                    returnValue = returnValueHandler;
                }
                return returnValue;
            })
            .then(function (pData) {
                if (defer_send) {
                    defer_send.resolve(pData);
                }
                if (eventName.indexOf('response-') > -1) {
                    //如果在response的event
                    //不再postMessage
                    // return Q.when(event.data.data);
                    return Q.when();
                }

                const p = postMessage({
                    type: '000',
                    eventName: `response-${eventName}`,
                    data: pData,
                    destWindow: srcWindow
                });
                return p;
            })
        ;

    });
};

const on = function (pParameter) {
    if (!pParameter) {
        pParameter = {};
    }

    const eventName = pParameter.eventName;
    const handler = pParameter.handler;
    if (!eventName) {
        return;
    }

    addEventEntity({
        eventName: eventName,
        handler: handler
    });
};

const createIFrame = function (pParameter) {
    if (!pParameter) {
        pParameter = {};
    }

    const $container = pParameter.$container || $('body');

    const defer = Q.defer();

    const $iframe = $('<iframe></iframe>');

    $iframe.load(function () {
        // const domIframe = $iframe.get(0);
        // const contentWindow = domIframe.contentWindow;
        // defer.resolve(contentWindow);

        const domIFrame = $iframe.get(0);
        defer.resolve({
            domIFrame: domIFrame
        });
    });

    _.each(pParameter, function (propertyValue, propertyName) {
        try {
            $iframe.attr(propertyName, propertyValue);
        } catch (ex) {
            // NL.eformat(ex);
        }
    });

    $container.append($iframe);

    return defer.promise;
};

const o = {};
o.init = init;
o.postMessage = postMessage;
o.on = on;
o.createIFrame = createIFrame;

module.exports = o;