const _ = require(`underscore`);

/**
 * NL类库
 *
 * @module nllib
 */


/**
 * 获取指定代码中，花括号代码块的列表
 *
 * @class nllib/src/js-code-analyze/get_hua_kuo_hao_block_list
 * @static
 *
 * @param {object} pParameter
 * @param {function} pParameter.code 代码
 * @return {array} 花括号代码块的列表
 *
 * <div>[</div>
 * <div style="padding-left:2em;">{</div>
 * <div style="padding-left:4em;">start_index:1,//左花括号的传入代码中的索引</div>
 * <div style="padding-left:4em;">end_index:100,//右花括号的传入代码中的索引</div>
 * <div style="padding-left:4em;">code:"",//左花括号和右花括号之间的代码，包括左花括号和右花括号</div>
 * <div style="padding-left:2em;">},</div>
 * <div>]</div>
 *
 * @demo src/js-code-analyze/get_hua_kuo_hao_block_list/1.0.0/demo/index.demo.js
 */

const get_hua_kuo_hao_block_list = function (pParameter) {
    if (!pParameter) pParameter = {};
    const code = pParameter.code || ``;

    // console.log(`code.length=`, code.length);

    const left_kuo_hao_index_list = [];
    const right_kuo_hao_index_list = [];
    for (let i = 0; i < code.length; i++) {
        const c = code[i];
        // console.log(c);
        if (c === `{`) {
            left_kuo_hao_index_list.push(i);
        }
        if (c === `}`) {
            right_kuo_hao_index_list.push(i);
        }
    }

    const left_length = left_kuo_hao_index_list.length;
    const right_length = right_kuo_hao_index_list.length;
    // console.log(`left_length=`, left_length);
    // console.log(`right_length=`, right_length);

    if (left_length !== right_length) {
        const msg = `get_block_list parameter error, code=001`;
        console.error(msg);
        console.trace();
        return [];
    }

    const block_list_001 = [];
    for (let i = 0; i < left_length; i++) {
        const left_index = left_kuo_hao_index_list[i];
        const right_index = right_kuo_hao_index_list[left_length - i - 1];

        const start_index = left_index;
        const end_index = right_index;
        const __code = code.substring(start_index, end_index + 1);
        block_list_001.push({
            start_index,
            end_index,
            code: __code,
        });
    }

    const block_list_002 = _.chain(block_list_001)
        .sortBy(function (m) {
            return m.start_index;
        })
        .value()
    ;

    return block_list_002;
};

module.exports = get_hua_kuo_hao_block_list;
