const _ = require(`underscore`);

const to_boolean = require(`../../../convert/to_boolean`);
const get_hua_kuo_hao_block_list = require(`../../get_hua_kuo_hao_block_list`);

/**
 * NL类库
 *
 * @module nllib
 */


/**
 * 根据属性名，获取属性值
 *
 * @class nllib/src/js-code-analyze/get_property_value_by_property_name
 * @static
 *
 * @param {object} pParameter
 * @param {string} pParameter.code 代码
 * @param {string} pParameter.property_name 属性名
 * @param {boolean} pParameter.enable_return_meta 是否返回meta数据，默认值为false
 * @return {object} 数据
 * <div>数据格式为：</div>
 * <div>{</div>
 * <div style="padding-left:2em;">property_name:``,//属性名</div>
 * <div style="padding-left:2em;">property_value:``,//属性值</div>
 * <div>}</div>
 *
 * @demo src/js-code-analyze/get_property_value_by_property_name/1.0.0/demo/index.demo.js
 */

const get_property_value_by_property_name = function (pParameter) {
    if (!pParameter) pParameter = {};
    const code = pParameter.code || ``;
    const property_name = pParameter.property_name || ``;
    const enable_return_meta = to_boolean(pParameter.enable_return_meta, false);

    const block_list = get_hua_kuo_hao_block_list({
        code,
    });
    // console.log(JSON.stringify(block_list, null, 4));
    // console.log(`block_list.length=`, block_list.length);

    const index_property_name = code.indexOf(property_name);
    // console.log(`index_property_name=`, index_property_name);

    const match_block = _.chain(block_list)
        .find(function (m) {
            const start_index = m.start_index;
            return start_index >= index_property_name;
        })
        .value()
    ;
    // console.log(JSON.stringify(match_block, null, 4));
    let property_value = ``;
    if (match_block) {
        property_value = match_block.code;
    }

    const d = {
        property_name,
        property_value,
    };
    if (enable_return_meta) {
        d.meta = {
            index_property_name,
            match_block,
            block_list,
            code,
        };
    }
    return d;
};

module.exports = get_property_value_by_property_name;
