const path = require('path');

const extend = require('extend');
const _ = require('underscore');
const log4js = require(`log4js`);
const colors = require('colors');

/**
 * NL类库
 *
 * @module nllib
 */

/**
 * 初始化日志
 *
 * @class nllib/src/log-helper/init
 * @static
 *
 * @param {object} pParameter
 *
 * @param {string} pParameter.path_logs 日志根目录，默认值为path.resolve(`./logs`)
 * @param {boolean} pParameter.enable_console 是否启用console，默认值为false
 *
 * @demo src/log-helper/init/1.0.0/demo/index.demo.js
 */
const init = function (pParameter) {
    const defaultParameter = {};
    defaultParameter.path_logs = path.resolve(`./logs`);
    defaultParameter.enable_console = false;

    const parameter = extend(true, {}, defaultParameter, pParameter);

    const path_logs = parameter.path_logs;
    const enable_console = parameter.enable_console;

    const original_appender_config = {};
    original_appender_config.type = `dateFile`;
    original_appender_config.filename = ``;
    // original_appender_config.maxLogSize = 10 * 1024 * 1024;//10mb
    original_appender_config.maxLogSize = 1024 * 1024;//1mb
    original_appender_config.compress = true;
    original_appender_config.daysToKeep = 31 * 6;//半年

    const config = {
        appenders: {
            file_info: extend({}, original_appender_config, {filename: path.join(path_logs, `info.log`)}),
            file_warn: extend({}, original_appender_config, {filename: path.join(path_logs, `warn.log`)}),
            file_error: extend({}, original_appender_config, {filename: path.join(path_logs, `error.log`)}),
            console: {type: `console`},
        },
        categories: {
            default: {
                appenders: [
                    `file_info`,
                    `console`,
                ],
                level: `info`
            },
            '[category-info-001]': {
                appenders: [
                    `file_info`,
                    `console`,
                ],
                level: `info`
            },
            '[category-warn-001]': {
                appenders: [
                    `file_warn`,
                    `console`,
                ],
                level: `info`
            },
            '[category-error-001]': {
                appenders: [
                    `file_error`,
                    `console`,
                ],
                level: `info`
            },
            '[category-info]': {
                appenders: [
                    `file_info`,
                ],
                level: `info`
            },
            '[category-warn]': {
                appenders: [
                    `file_warn`,
                ],
                level: `info`
            },
            '[category-error]': {
                appenders: [
                    `file_error`,
                ],
                level: `info`
            },
        },
    };
    log4js.configure(config);

    const get_category = function ({level = `info`, enable_console = false}) {
        const s_001 = `category-${level}`;

        let s_002;
        if (enable_console) {
            s_002 = `${s_001}-001`;
        } else {
            s_002 = s_001;
        }

        const s_003 = `[${s_002}]`;
        return s_003;
    };

    const __console = console;
    console.log = function () {
        const category = get_category({
            level: `info`,
            enable_console: enable_console,
        });
        const logger = log4js.getLogger(category);

        const list = _.toArray(arguments);
        const msg = list.join(` `);
        logger.info(msg);
    };

    console.warn = function (msg) {
        const category = get_category({
            level: `warn`,
            enable_console: enable_console,
        });
        const logger = log4js.getLogger(category);
        logger.warn(msg);
    };

    console.error = function (msg) {
        const category = get_category({
            level: `error`,
            enable_console: enable_console,
        });
        const logger = log4js.getLogger(category);
        logger.error(msg);
    };
};

module.exports = init;

//参考
//https://github.com/log4js-node/log4js-node
//https://log4js-node.github.io/log4js-node/dateFile.html
//https://log4js-node.github.io/log4js-node/file.html
//https://log4js-node.github.io/log4js-node/console.html
//https://stackoverflow.com/questions/44645006/how-to-log-the-console-logs-into-the-file-using-log4js
