//<editor-fold desc="node core modules">
// const path = require('path');
//</editor-fold>

//<editor-fold desc="npm modules">
// const fs = require('fs-extra');
const extend = require('extend');
const Q = require('q');
const _ = require('underscore');
// const moment = require('moment');
// const URI = require('urijs');
const $ = require(`jquery`);
//</editor-fold>


const get_platform_information = require(`../uri/get_platform_information`);
const join = require(`../uri/join`);
const to_object = require(`../json/to_object`);
const get_token = require(`./get_token`);

const request_webApiType_v2 = function (pParameter) {
    const defaultParameter = {};
    defaultParameter.type = 'post';//post|get
    defaultParameter.contentType = 'application/json;charset=utf-8';//application/json;charset=utf-8|application/x-www-form-urlencoded
    defaultParameter.dataType = 'json';//json|jsonp
    defaultParameter.urlSuffix = '';
    defaultParameter.data = {};
    defaultParameter.headers = {};
    defaultParameter.timeout = 30 * 1000;
    defaultParameter.processData = true;
    defaultParameter.getUrl = function () {
        const getUrlPrefix = function () {
            const platformInformation = get_platform_information({webApiType: 'v2'});
            const returnUrlPrefix = platformInformation.urlPrefix;
            return returnUrlPrefix;
        };
        const urlPrefix = getUrlPrefix();

        const url = join(urlPrefix, parameter.urlSuffix);
        return url;
    };

    defaultParameter.v2 = {};

    defaultParameter.sso = {};

    const parameter = extend(true, {}, defaultParameter, pParameter);

    const getType = function () {
        const type = parameter.type;
        return type;
    };
    const type = getType();

    const getContentType = function () {
        const contentType = parameter.contentType;
        return contentType;
    };
    const contentType = getContentType();

    const getDataType = function () {
        return parameter.dataType;
    };
    const dataType = getDataType();

    const getUrl = parameter.getUrl;
    const url = getUrl();

    const getData = function () {
        let data;
        switch (contentType) {
            case 'application/json;charset=utf-8':
                if (parameter.type === 'post') {
                    data = JSON.stringify(parameter.data)
                } else {
                    data = parameter.data;
                }
                break;

            default:
                data = parameter.data;
                break;
        }
        return data;
    };
    const data = getData(parameter);

    const getHeaders = function () {
        const headers = extend(true, {}, parameter.headers);

        const _token = get_token();
        const token = headers.authorization || _token;
        if (token) {
            headers.authorization = token;
        }
        // headers.requestId = get_uuid();

        return headers;
    };
    const headers = getHeaders();

    const timeout = parameter.timeout;

    const processData = parameter.processData;

    const error = function (jqXHR, textStatus) {
        const process_textStatus_error = function () {

            const get_response_data = function () {
                let response_data;

                const responseText = jqXHR.responseText;
                if (!responseText) {
                    return response_data
                }
                response_data = to_object(responseText);
                return response_data;
            }

            const log_error_msg = function () {
                console.error(`系统出现未知错误，请联系管理员！`);
                console.error(jqXHR);

                const response_data = get_response_data();
                console.error(response_data);
            }

            const process_status_403 = function () {

                const response_data = get_response_data();

                const check = function () {
                    let flag = false;

                    if (!global.NL) return flag;
                    if (!global.NL.model) return flag;
                    if (!global.NL.model.global_config) return flag;
                    if (!global.NL.model.global_config.request_webApiType_v2) return flag;
                    if (!global.NL.model.global_config.request_webApiType_v2.process_status_403) return flag;
                    if (!_.isFunction(global.NL.model.global_config.request_webApiType_v2.process_status_403)) return flag;

                    flag = true;
                    return flag;
                }

                const flag = check();
                if (flag) {
                    global.NL.model.global_config.request_webApiType_v2.process_status_403({
                        jqXHR: jqXHR,
                        textStatus: textStatus,
                        response_data: response_data,
                    });
                    return;
                }

                log_error_msg();
            };

            const process_status_default = function () {

                const response_data = get_response_data();

                const check = function () {
                    let flag = false;

                    if (!global.NL) return flag;
                    if (!global.NL.model) return flag;
                    if (!global.NL.model.global_config) return flag;
                    if (!global.NL.model.global_config.request_webApiType_v2) return flag;
                    if (!global.NL.model.global_config.request_webApiType_v2.process_status_default) return flag;
                    if (!_.isFunction(global.NL.model.global_config.request_webApiType_v2.process_status_default)) return flag;

                    flag = true;
                    return flag;
                }

                const flag = check();
                if (flag) {
                    global.NL.model.global_config.request_webApiType_v2.process_status_default({
                        jqXHR: jqXHR,
                        textStatus: textStatus,
                        response_data: response_data,
                    });
                    return;
                }

                log_error_msg();
            };

            const status = jqXHR.status;
            switch (status) {
                case 403:
                    process_status_403();
                    break;

                default:
                    process_status_default();
                    break;
            }

        };

        switch (textStatus) {
            case 'timeout':
                console.error(`请求超时！`);
                break;

            case 'error':
                process_textStatus_error();
                break;

            default :
                console.error(`系统出现未知错误，请联系管理员！`);
                break;
        }
    };

    const ajaxOption = {
        type: type,
        contentType: contentType,
        dataType: dataType,
        url: url,
        data: data,
        headers: headers,
        timeout: timeout,
        processData: processData,
        error: error
    };

    return Q.when()
        .then(function () {
            return $.ajax(ajaxOption);
        })
        .then(function (data) {
            return Q.resolve(data);
        })
        .catch(function (ex) {
            console.error(ex);
            if (ex) {
                if (ex.stack) {
                    console.error(ex.stack);
                }
            }
            return Q.reject(ex);
        })
        ;
};

module.exports = request_webApiType_v2;
