const extend = require('extend');
const Q = require('q');
const _ = require('underscore');
const axios = require(`axios`);
const get_uuid = require(`..//common/get_uuid`);

const request_webApiType_v3 = function (pParameter) {
    const defaultParameter = {};
    defaultParameter.method = 'post';//post|get
    defaultParameter.data = {};
    defaultParameter.headers = {};
    defaultParameter.timeout = 10 * 1000;
    defaultParameter.maxRedirects = 1;

    defaultParameter.request_id = {
        enable: true,
        property_name: `requestId`,
        property_value: ``,
    };

    defaultParameter.get_url = function () {
        const msg = 'request_webApiType_v3 parameter error, code=001';
        console.error(msg);
    };
    defaultParameter.get_token = function () {
        const msg = 'request_webApiType_v3 parameter error, code=002';
        console.error(msg);
    };

    let requestId;

    const parameter = extend(true, {}, defaultParameter, pParameter);

    const method = parameter.method;

    const get_url = parameter.get_url;
    const url = get_url();
    if (!url) {
        const msg = 'request_webApiType_v3 parameter error, code=101';
        console.error(msg);
        return Q.reject({msg: msg});
    }

    const data = parameter.data;

    const get_headers = function () {
        const headers = extend(true, {}, parameter.headers);

        let __token;
        const get_token = parameter.get_token;
        if (_.isFunction(get_token)) {
            __token = get_token();
        }

        const token = parameter.headers.authorization || __token;

        if (token) {
            headers.authorization = token;
        }

        if (parameter.request_id.enable) {
            const property_name = parameter.request_id.property_name;
            const property_value = parameter.request_id.property_value;
            if (property_value) {
                requestId = property_value;
            } else {
                requestId = get_uuid();
            }
            headers[property_name] = requestId;
        }

        return headers;
    };
    const headers = get_headers();

    const timeout = parameter.timeout;

    const maxRedirects = parameter.maxRedirects;

    const options = {
        method: method,
        url: url,
        data: data,
        headers: headers,
        timeout: timeout,
        maxRedirects: maxRedirects,
    };

    if (global) {
        if (global.process) {
            if (global.process.env) {
                if (global.process.env.nllib_enable_fiddler) {
                    options.proxy = {
                        host: '127.0.0.1',
                        port: 8888,
                    };
                }
            }
        }
    }

    return Q.when()
        .then(function () {
            return axios(options);
        })
        .then(function (res) {
            return Q.resolve(res.data);
        })
        .catch(function (ex) {
            console.error(ex);
            if (ex) {
                if (ex.stack) {
                    console.error(ex.stack);
                }
            }
            return Q.reject(ex);
        })
        ;
};

module.exports = request_webApiType_v3;
