const path = require(`path`);
const fs = require(`fs-extra`);

const extend = require(`extend`);
const _ = require(`underscore`);
const shell = require(`shelljs`);

const get_file_list = function (pParameter) {
    const defaultParameter = {};
    defaultParameter.path_dir = ``;
    defaultParameter.enable_recurse_subdirectories = true;
    defaultParameter.enable_read_file_content = false;

    const parameter = extend({}, defaultParameter, pParameter);

    const path_dir = parameter.path_dir;
    const enable_recurse_subdirectories = parameter.enable_recurse_subdirectories;
    const enable_read_file_content = parameter.enable_read_file_content;

    if (!fs.existsSync(path_dir)) {
        const msg = `get_file_list parameter error, code=001`;
        console.error(msg);
        console.trace();
        return;
    }

    const process_enable_recurse_subdirectories_true = function () {
        const list_001 = shell.find(path_dir);
        const list_002 = _.chain(list_001)
            .filter(function (m) {
                const s = fs.statSync(m);
                return s.isFile();
            })
            .value()
        ;
        const list_003 = _.chain(list_002)
            .map(function (m) {
                const path_file = m;
                const path_dir = path.dirname(path_file);
                const file_name = path.basename(path_file);
                const file_ext_name = path.extname(path_file);
                let file_content = ``;
                if (enable_read_file_content) {
                    if (fs.existsSync(path_file)) {
                        file_content = fs.readFileSync(path_file, `utf-8`)
                    }
                }
                return {
                    file_name,
                    file_ext_name,
                    file_content,
                    path_file,
                    path_dir,
                };
            })
            .value()
        ;
        return list_003;
    };

    const process_enable_recurse_subdirectories_false = function () {
        const list_001 = fs.readdirSync(path_dir);
        // console.log(JSON.stringify(list_001, null, 4));
        const list_002 = _.chain(list_001)
            .map(function (m) {
                const file_name = m;
                const path_file = path.join(path_dir, file_name);
                return path_file;
            })
            .filter(function (m) {
                const s = fs.statSync(m);
                return s.isFile();
            })
            .value();
        const list_003 = _.chain(list_002)
            .map(function (m) {
                const path_file = m;
                const path_dir = path.dirname(path_file);
                const file_name = path.basename(path_file);
                const file_ext_name = path.extname(path_file);
                let file_content = ``;
                if (enable_read_file_content) {
                    if (fs.existsSync(path_file)) {
                        file_content = fs.readFileSync(path_file, `utf-8`)
                    }
                }
                return {
                    file_name,
                    file_ext_name,
                    file_content,
                    path_file,
                    path_dir,
                };
            })
            .value()
        ;
        // console.log(JSON.stringify(list_003, null, 4));
        return list_003;
    };

    if (enable_recurse_subdirectories) return process_enable_recurse_subdirectories_true();
    if (!enable_recurse_subdirectories) return process_enable_recurse_subdirectories_false();
};

module.exports = get_file_list;
