const path = require(`path`);
const fs = require(`fs-extra`);
const moment = require(`moment`);

const to_boolean = require(`../../../convert/to_boolean`);

/**
 * NL类库
 *
 * @module nllib
 */

/**
 * 初始化目录
 *
 * @class nllib/src/path/init_directory
 * @static
 *
 * @param {object} pParameter
 *
 * @param {string} pParameter.type 初始化类型
 * <div>000：初始化指定目录，不会清空原有目录下的所有文件</div>
 * <div>100：先清空原有目录下的所有文件，再进行初始化</div>
 * <div>默认值为000</div>
 * @param {string} pParameter.path_dir 待初始化的目录，默认值为path.resolve(`./`)
 * @param {boolean} pParameter.enable_log 是否输出日志，默认值为true
 *
 * @demo src/path/init_directory/1.0.1/demo/index.demo.js
 */
const init_directory = function (pParameter) {
    if (!pParameter) pParameter = {};
    const type = pParameter.type || `000`;
    const path_dir = pParameter.path_dir || path.resolve(`./`);
    const enable_log = to_boolean(pParameter.enable_log, true);

    const process_type_000 = function () {
        const path_file_init_directory_md = path.join(path_dir, `init-directory.md`);

        fs.outputFileSync(path_file_init_directory_md, moment().format(`YYYY-MM-DD HH:mm:ss`));

        if (enable_log) console.log(`nllib/src/path/init_directory:`, `文件构建成功`, path_file_init_directory_md);

        return {
            path_dir,
            path_file_init_directory_md,
        };
    };

    const process_type_100 = function () {
        process_type_000();

        fs.removeSync(path_dir);
        if (enable_log) console.log(`nllib/src/path/init_directory:`, `清空目录成功`, path_dir);

        return process_type_000();
    };

    if (type === `100`) return process_type_100();
    return process_type_000();
};

module.exports = init_directory;
