// const path = require(`path`);
// const fs = require(`fs-extra`);
const extend = require(`extend`);
// const Q = require(`q`);
const _ = require(`underscore`);
// const moment = require(`moment`);

/**
 * NL类库
 *
 * @module nllib
 */

/**
 * 获取第一个左孩子结点实体，传入结点实体
 *
 * @class nllib/src/tree-helper/__get_first_child_node_001
 * @static
 *
 * @param {object} pParameter
 *
 * @param {array} pParameter.tree 一维数组，通过property_name_for_children包含树状数据
 * @param {string} pParameter.property_name_for_children children的属性名称
 * @return {object} 第一个左孩子结点实体
 *
 * @demo src/tree-helper/get_first_child_node/demo/get_first_child_node.demo.js
 */


const __get_first_child_node_by_node = function (pParameter) {
    if (!pParameter) pParameter = {};
    const node = pParameter.node;
    const property_name_for_children = pParameter.property_name_for_children;

    if (!node) {
        const msg = `__get_first_child_node_by_node parameter error, code=001`;
        console.error(msg);
        console.trace();
        return;
    }

    const children = node[property_name_for_children];
    if (!_.isArray(children)) {
        return node;
    }

    if (children.length <= 0) {
        return node;
    }

    const next_node = children[0];
    return __get_first_child_node_by_node({
        node: next_node,
        property_name_for_children: property_name_for_children,
    });
};

const __get_first_child_node_by_node_list = function (pParameter) {
    if (!pParameter) pParameter = {};
    const node_list = pParameter.node_list;
    const property_name_for_children = pParameter.property_name_for_children;

    if (!_.isArray(node_list)) {
        return;
    }

    if (node_list.length <= 0) {
        return;
    }

    const node = node_list[0];
    return __get_first_child_node_by_node({
        node,
        property_name_for_children,
    });

};

const get_first_child_node = function (pParameter) {
    if (!pParameter) pParameter = {};
    const tree = pParameter.tree;
    const property_name_for_children = pParameter.property_name_for_children;

    if (_.isArray(tree)) {
        return __get_first_child_node_by_node_list({
            node_list: tree,
            property_name_for_children,
        });
    }

    return __get_first_child_node_by_node({
        node: tree,
        property_name_for_children,
    });
};

module.exports = get_first_child_node;
