/**
 * NL类库(deprecated)
 *
 * @module nllib-deprecated
 */

/**
 * list转换成tree
 *
 * @class nllib/src/tree-helper/list_to_tree/1.0.0
 * @static
 *
 * @param {object} pParameter
 *
 * @param {array} pParameter.list 一维数组
 * @param {string} pParameter.property_name_for_id id的属性名称
 * @param {string} pParameter.property_name_for_parent_id parent_id的属性名称
 * @param {string} pParameter.property_name_for_children children的属性名称
 * @param {string} pParameter.check_is_root_parent_id 检查当前传入的parent是否为根结点的parent_id，即用于定义根结点的parent_id的值
 * <div>方法签名：function ({parent_id}) {}</div>
 * <div>默认方法：function ({parent_id}) {return !parent_id;}</div>
 *
 * @return {array} 一维数组，通过property_name_for_children包含树状数据
 *
 * @demo src/tree-helper/list_to_tree/1.0.0/demo/list_to_tree.demo.js
 */

// const extend = require(`extend`);
// const Q = require(`q`);
const _ = require(`underscore`);
// const moment = require(`moment`);

const __check_is_root_parent_id = function ({parent_id}) {
    return !parent_id;
};

const list_to_tree = function (pParameter) {
    if (!pParameter) pParameter = {};
    const list = pParameter.list;
    const property_name_for_id = pParameter.property_name_for_id || `id`;
    const property_name_for_parent_id = pParameter.property_name_for_parent_id || `parent_id`;
    const property_name_for_children = pParameter.property_name_for_children || `children`;
    const check_is_root_parent_id = _.isFunction(pParameter.check_is_root_parent_id) ? pParameter.check_is_root_parent_id : __check_is_root_parent_id;

    if (!_.isArray(list)) {
        const msg = `list_to_tree parameter error, code=001`;
        console.error(msg);
        console.trace();
        return [];
    }

    _.each(list, function (entity) {
        const id = entity[property_name_for_id];

        const children = _.chain(list)
            .filter(function (m) {
                const parent_id = m[property_name_for_parent_id];
                return id === parent_id;
            })
            .value()
        ;
        if (children.length <= 0) return;

        entity[property_name_for_children] = children;
    });

    const tree = _.chain(list)
        .filter(function (m) {
            const parent_id = m[property_name_for_parent_id];
            return check_is_root_parent_id({
                parent_id,
            });
        })
        .value()
    ;

    return tree;
};

module.exports = list_to_tree;
