/**
 * NL类库
 *
 * @module nllib
 */

/**
 * tree迭代
 *
 * @class nllib/src/tree-helper/tree_iteration
 * @static
 *
 * @param {object} pParameter
 *
 * @param {array} pParameter.tree 一维数组，通过property_name_for_children包含树状数据
 * @param {string} pParameter.property_name_for_children children的属性名称
 * @param {function} pParameter.handler 迭代回调，方法签名：function(args) {}
 * <div>args</div>
 * <div>args.entity：当前迭代的数据实体</div>
 *
 * @demo src/tree-helper/tree_iteration/demo/tree_iteration.demo.js
 */

// const path = require(`path`);
// const fs = require(`fs-extra`);

// const extend = require(`extend`);
// const Q = require(`q`);
const _ = require(`underscore`);
// const moment = require(`moment`);

const tree_iteration = function (pParameter) {
    if (!pParameter) pParameter = {};

    const tree = pParameter.tree;
    const property_name_for_children = pParameter.property_name_for_children || `children`;
    const handler = pParameter.handler;

    if (!_.isArray(tree)) return;
    if (!_.isFunction(handler)) return;

    _.each(tree, function (entity) {
        handler({entity});
        const children = entity[property_name_for_children];
        if (!_.isArray(children)) return;
        tree_iteration({
            tree: children,
            property_name_for_children: property_name_for_children,
            handler: handler,
        });
    });
};

module.exports = tree_iteration;
