//<editor-fold desc="node core modules">
const path = require(`path`);
const url = require('url');
const crypto = require('crypto');
//</editor-fold>

//<editor-fold desc="npm modules">
const extend = require('extend');
const Q = require('q');

const rp = require('request-promise');

//</editor-fold>

module.exports = function (pParameter) {
    const defaultParameter = {};
    defaultParameter.verbose = false;
    defaultParameter.accessKeyId = '';
    defaultParameter.accessKeySecret = '';
    defaultParameter.url = '';
    defaultParameter.method = 'POST';
    defaultParameter.body = '';

    const parameter = extend(true, {}, defaultParameter, pParameter);

    const verbose = parameter.verbose;
    const accessKeyId = parameter.accessKeyId;
    const accessKeySecret = parameter.accessKeySecret;
    const __url = parameter.url;
    const __method = parameter.method;
    const __body = parameter.body;

    if (!accessKeyId) {
        const msg = '__request parameter error, code=001';
        console.error(msg);
        return Q.reject({msg: msg});
    }
    if (!accessKeySecret) {
        const msg = '__request parameter error, code=002';
        console.error(msg);
        return Q.reject({msg: msg});
    }
    if (!__url) {
        const msg = '__request parameter error, code=003';
        console.error(msg);
        return Q.reject({msg: msg});
    }

    //不知道为什么一定要全部大写才可以
    const method = __method.toUpperCase();

    const date = new Date().toUTCString()
// 这里填写AK和请求
    const ak_id = accessKeyId;
    const ak_secret = accessKeySecret;
    const options = {
        url: __url,
        method: method,
        body: __body,
        headers: {
            'accept': 'application/json',
            'content-type': 'application/json',
            'date': date,
            'Authorization': ''
        },
        // json:true,//不能够传入这个参数，阿里云会报403
    };
// 这里填写AK和请求
    const md5 = function (buffer) {
        let hash;
        hash = crypto.createHash('md5');
        hash.update(buffer);
        return hash.digest('base64');
    };
    const sha1 = function (stringToSign, secret) {
        let signature;
        return signature = crypto.createHmac('sha1', secret).update(stringToSign).digest().toString('base64');
    };
// step1: 组stringToSign [StringToSign = #{method}\\n#{accept}\\n#{data}\\n#{contentType}\\n#{date}\\n#{action}]
    const body = options.body || '';
    let bodymd5;
    if (body === void 0 || body === '') {
        bodymd5 = body;
    } else {
        bodymd5 = md5(new Buffer(body));
    }
    if (verbose) console.log(bodymd5);
    const stringToSign = options.method + "\n" + options.headers.accept + "\n" + bodymd5 + "\n" + options.headers['content-type'] + "\n" + options.headers.date + "\n" + url.parse(options.url).path;
    if (verbose) console.log("step1-Sign string:", stringToSign);
// step2: 加密 [Signature = Base64( HMAC-SHA1( AccessSecret, UTF-8-Encoding-Of(StringToSign) ) )]
    const signature = sha1(stringToSign, ak_secret);
// console.log("step2-signature:", signature);
// step3: 组authorization header [Authorization =  Dataplus AccessKeyId + ":" + Signature]
    const authHeader = "Dataplus " + ak_id + ":" + signature;
    if (verbose) console.log("step3-authorization Header:", authHeader);
    options.headers.Authorization = authHeader;
    if (verbose) console.log('authHeader', authHeader);

// step4: send request
//     function callback(error, response, body) {
//         if (error) {
//             console.log("error", error)
//         }
//         console.log("step4-response body:", response.statusCode, body)
//     }

// request(options, callback);
    return rp(options).then(function (json) {
        const data = JSON.parse(json);
        return Q.resolve(data);
    });

};