//<editor-fold desc="node core modules">
// const path = require('path');
//</editor-fold>

//<editor-fold desc="npm modules">
// const fs = require('fs-extra');
const extend = require('extend');
const Q = require('q');
// const _ = require('underscore');
// const moment = require('moment');
// const URI = require('urijs');
//</editor-fold>

const __request = require(`../../../common/backend/__request`);

/**
 * NL类库
 *
 * @module NL
 */

/**
 * 阿里云image recognition客户端
 * <div>
 * <span class="tag margin-right">不支持浏览器环境</span>
 * <span class="tag margin-right">支持node环境</span>
 * </div>
 * <br>官方文档：https://help.aliyun.com/knowledge_detail/53540.html?spm=a2c4g.11186623.6.550.73866fadA2VgsI
 *
 * @class NL.systemDocking.aliyun.imageRecognition.backend.client
 * @constructor
 *
 * @param {object} pInitParameter
 * @param {string} pInitParameter.accessKeyId 阿里云的accessKeyId
 * @param {string} pInitParameter.accessKeySecret 阿里云的accessKeySecret
 
 *
 * @return {object} NL.systemDocking.aliyun.imageRecognition.backend.client实例
 *
 */
module.exports = function (pInitParameter) {
    const __this = this;

    const defaultInitParameter = {};
    defaultInitParameter.accessKeyId = ``;
    defaultInitParameter.accessKeySecret = ``;

    const initParameter = extend(true, {}, defaultInitParameter, pInitParameter);

    const accessKeyId = initParameter.accessKeyId;
    const accessKeySecret = initParameter.accessKeySecret;

    if (!accessKeyId) {
        const msg = 'NL.systemDocking.aliyun.imageRecognition.backend.client parameter error, code=001';
        console.error(msg);
        return Q.reject({msg: msg});
    }
    if (!accessKeySecret) {
        const msg = 'NL.systemDocking.aliyun.imageRecognition.backend.client parameter error, code=002';
        console.error(msg);
        return Q.reject({msg: msg});
    }


    /**
     * 图像打标签
     * <br>官方文档：https://help.aliyun.com/knowledge_detail/53540.html?spm=a2c4g.11186623.6.550.c5c17044WWSd1B
     *
     * @method image_tag
     *
     * @param {object} pParameter
     * @param {int} pParameter.type
     * @param {string} pParameter.image_url
     * @param {string} pParameter.content
     *
     * @return {object} promise
     *
     * @demo src/system-docking/aliyun/image-recognition/backend/client/demo/image_tag.demo.js
     */
    const image_tag = function (pParameter) {
        if (!pParameter) {
            pParameter = {};
        }

        const type = pParameter.type;
        const image_url = pParameter.image_url;
        const content = pParameter.content;

        return Q.when()
            .then(function () {
                return __request({
                    accessKeyId: accessKeyId,
                    accessKeySecret: accessKeySecret,
                    url: `https://dtplus-cn-shanghai.data.aliyuncs.com/image/tag`,
                    method: `post`,
                    body: JSON.stringify({
                        type: type,
                        image_url: image_url,
                        content: content,
                    }),
                });
            })
            .catch(function (ex) {
                console.error(ex);
                if (ex) {
                    if (ex.stack) {
                        console.error(ex.stack);
                    }
                }
                return Q.reject(ex);
            })
            ;
    };

    __this.image_tag = image_tag;

};