//<editor-fold desc="node core modules">
// const path = require('path');
//</editor-fold>

//<editor-fold desc="npm modules">
// const fs = require('fs-extra');
const extend = require('extend');
const Q = require('q');
// const _ = require('underscore');
// const moment = require('moment');
// const URI = require('urijs');
const ALY = require('@ningkyolei/aliyun-sdk');
//</editor-fold>

/**
 * NL类库
 *
 * @module NL
 */

/**
 * 阿里云sts客户端
 * <div>
 * <span class="tag margin-right">不支持浏览器环境</span>
 * <span class="tag margin-right">支持node环境</span>
 * </div>
 * <br>官方文档：https://help.aliyun.com/document_detail/28756.html?spm=a2c4g.11186623.6.673.6a3a4be5rLDlh5
 *
 * @class NL.systemDocking.aliyun.sts.backend.client
 * @constructor
 *
 * @param {object} pInitParameter
 * @param {string} pInitParameter.accessKeyId 阿里云的accessKeyId
 * @param {string} pInitParameter.accessKeySecret 阿里云的accessKeySecret
 *
 * @return {object} NL.systemDocking.aliyun.sts.backend.client实例
 *
 */
module.exports = function (pInitParameter) {
    const __this = this;

    const defaultInitParameter = {};
    defaultInitParameter.accessKeyId = ``;
    defaultInitParameter.accessKeySecret = ``;

    const initParameter = extend(true, {}, defaultInitParameter, pInitParameter);

    const accessKeyId = initParameter.accessKeyId;
    const accessKeySecret = initParameter.accessKeySecret;

    if (!accessKeyId) {
        const msg = 'NL.systemDocking.aliyun.sts.backend.client parameter error, code=001';
        console.error(msg);
        return Q.reject({msg: msg});
    }
    if (!accessKeySecret) {
        const msg = 'NL.systemDocking.aliyun.sts.backend.client parameter error, code=002';
        console.error(msg);
        return Q.reject({msg: msg});
    }

    const __client = new ALY.STS({
        accessKeyId: accessKeyId,
        secretAccessKey: accessKeySecret,
        endpoint: 'https://sts.aliyuncs.com',
        apiVersion: '2015-04-01'
    });

    const assumeRole = function (pParameter) {
        if (!pParameter) {
            pParameter = {};
        }

        const RoleArn = pParameter.RoleArn;
        const RoleSessionName = pParameter.RoleSessionName;

        if (!RoleArn) {
            const msg = 'assumeRole parameter error, code=001';
            console.error(msg);
            return Q.reject({msg: msg});
        }
        if (!RoleSessionName) {
            const msg = 'assumeRole parameter error, code=002';
            console.error(msg);
            return Q.reject({msg: msg});
        }

        const defer = Q.defer();
        // 构造AssumeRole请求
        __client.assumeRole(
            {
                Action: 'AssumeRole',
                // 指定角色Arn
                RoleArn: RoleArn,
                //设置Token的附加Policy，可以在获取Token时，通过额外设置一个Policy进一步减小Token的权限；
                //Policy: '{"Version":"1","Statement":[{"Effect":"Allow", "Action":"*", "Resource":"*"}]}',
                //设置Token有效期，可选参数，默认3600秒；
                //DurationSeconds: 3600,
                RoleSessionName: RoleSessionName,
            },
            function (err, data) {
                if (err) {
                    defer.reject(err);
                    return;
                }
                defer.resolve(data);
            });
        return defer.promise;
    };

    __this.__client = __client;

    /**
     * assumeRole
     * <br>官方文档：https://help.aliyun.com/document_detail/28763.html?spm=a2c4g.11186623.6.680.763f25fb4Fh4Qx
     *
     * @method assumeRole
     *
     * @param {object} pParameter
     * @return {object} promise
     *
     * @demo src/system-docking/aliyun/sts/backend/client/demo/assumeRole.demo.js
     */
    __this.assumeRole = assumeRole;
};