//<editor-fold desc="node core modules">
// const path = require('path');
//</editor-fold>

//<editor-fold desc="npm modules">
// const fs = require('fs-extra');
const extend = require('extend');
const Q = require('q');
// const _ = require('underscore');
// const moment = require('moment');
// const URI = require('urijs');
const TableStore = require(`tablestore`);
//</editor-fold>

/**
 * NL类库
 *
 * @module NL
 */

/**
 * 阿里云table store客户端
 * <div>
 * <span class="tag margin-right">不支持浏览器环境</span>
 * <span class="tag margin-right">支持node环境</span>
 * </div>
 * <br>官方文档：https://help.aliyun.com/document_detail/27304.html?spm=a2c4g.11186623.6.733.4fa3223bvwCq8w
 * <br>官方sdk：https://github.com/aliyun/aliyun-tablestore-nodejs-sdk
 *
 * @class NL.systemDocking.aliyun.tableStore.backend.client
 * @constructor
 *
 * @param {object} pInitParameter
 * @param {string} pInitParameter.accessKeyId 阿里云的accessKeyId
 * @param {string} pInitParameter.accessKeySecret 阿里云的accessKeySecret
 * @param {string} pInitParameter.endPoint 实例访问地址
 * @param {string} pInitParameter.instanceName TableStore实例名称
 *
 * @return {object} NL.systemDocking.aliyun.tableStore.backend.client实例
 *
 */
module.exports = function (pInitParameter) {
    const __this = this;

    const defaultInitParameter = {};
    defaultInitParameter.accessKeyId = ``;
    defaultInitParameter.accessKeySecret = ``;
    defaultInitParameter.endPoint = ``;
    defaultInitParameter.instanceName = ``;

    const initParameter = extend(true, {}, defaultInitParameter, pInitParameter);

    const accessKeyId = initParameter.accessKeyId;
    const accessKeySecret = initParameter.accessKeySecret;
    const endPoint = initParameter.endPoint;
    const instanceName = initParameter.instanceName;

    if (!accessKeyId) {
        const msg = 'NL.systemDocking.aliyun.tableStore.backend.client parameter error, code=001';
        console.error(msg);
        return Q.reject({msg: msg});
    }
    if (!accessKeySecret) {
        const msg = 'NL.systemDocking.aliyun.tableStore.backend.client parameter error, code=002';
        console.error(msg);
        return Q.reject({msg: msg});
    }
    if (!endPoint) {
        const msg = 'NL.systemDocking.aliyun.tableStore.backend.client parameter error, code=003';
        console.error(msg);
        return Q.reject({msg: msg});
    }
    if (!instanceName) {
        const msg = 'NL.systemDocking.aliyun.tableStore.backend.client parameter error, code=004';
        console.error(msg);
        return Q.reject({msg: msg});
    }

    const __client = new TableStore.Client({
        accessKeyId: accessKeyId,
        secretAccessKey: accessKeySecret,
        endpoint: endPoint,
        instancename: instanceName,
    });

    const getRange = function (pParameter) {
        const defer = Q.defer();
        __client.getRange(pParameter, function (err, data) {
            if (err) {
                defer.reject(err);
                return;
            }

            defer.resolve(data);
        });
        return defer.promise;
    };

    const batchWriteRow = function (pParameter) {
        const defer = Q.defer();
        __client.batchWriteRow(pParameter, function (err, data) {
            if (err) {
                defer.reject(err);
                return;
            }

            defer.resolve(data);
        });
        return defer.promise;
    };

    __this.__client = __client;

    /**
     * batchWriteRow
     * <br>官方文档：https://help.aliyun.com/document_detail/27311.html?spm=a2c4g.11186623.6.740.63ee7a3bKokD3b
     *
     * @method batchWriteRow
     *
     * @param {object} pParameter
     * @return {object} promise
     *
     * @demo src/system-docking/aliyun/table-store/backend/client/demo/batchWriteRow.demo.js
     */
    __this.batchWriteRow = batchWriteRow;

    /**
     * getRange
     * <br>官方文档：https://help.aliyun.com/document_detail/27309.html?spm=a2c4g.11186623.2.11.26ac20fd3y6H0r#reference3923
     *
     * @method getRange
     *
     * @param {object} pParameter
     * @return {object} promise
     *
     * @demo src/system-docking/aliyun/table-store/backend/client/demo/getRange.demo.js
     */
    __this.getRange = getRange;
};