//<editor-fold desc="node core modules">
// const path = require('path');
const querystring = require('querystring');
const https = require('https');
const crypto = require('crypto');
//</editor-fold>

//<editor-fold desc="npm modules">
// const fs = require('fs-extra');
const extend = require('extend');
const Q = require('q');
// const _ = require('underscore');
// const moment = require('moment');
// const URI = require('urijs');
const {RPCClient} = require('@alicloud/pop-core');
//</editor-fold>

/**
 * NL类库
 *
 * @module NL
 */

/**
 * 阿里云vod客户端
 * <div>
 * <span class="tag margin-right">不支持浏览器环境</span>
 * <span class="tag margin-right">支持node环境</span>
 * </div>
 * <br>官方文档：https://help.aliyun.com/document_detail/60574.html?spm=a2c4g.11186623.6.660.17d55157sOyrqM
 *
 * @class NL.systemDocking.aliyun.vod.backend.client
 * @constructor
 *
 * @param {object} pInitParameter
 * @param {string} pInitParameter.accessKeyId 阿里云的accessKeyId
 * @param {string} pInitParameter.accessKeySecret 阿里云的accessKeySecret
 *
 * @return {object} NL.systemDocking.aliyun.vod.backend.client实例
 *
 */
module.exports = function (pInitParameter) {
    const __this = this;

    const defaultInitParameter = {};
    defaultInitParameter.accessKeyId = ``;
    defaultInitParameter.accessKeySecret = ``;
    defaultInitParameter.endpoint = `https://vod.cn-shanghai.aliyuncs.com`;
    defaultInitParameter.apiVersion = `2017-03-21`;

    const initParameter = extend(true, {}, defaultInitParameter, pInitParameter);

    const accessKeyId = initParameter.accessKeyId;
    const accessKeySecret = initParameter.accessKeySecret;
    const endpoint = initParameter.endpoint;
    const apiVersion = initParameter.apiVersion;

    if (!accessKeyId) {
        const msg = 'NL.systemDocking.aliyun.vod.backend.client parameter error, code=001';
        console.error(msg);
        return Q.reject({msg: msg});
    }
    if (!accessKeySecret) {
        const msg = 'NL.systemDocking.aliyun.vod.backend.client parameter error, code=002';
        console.error(msg);
        return Q.reject({msg: msg});
    }

    const __client = new RPCClient({
        accessKeyId: accessKeyId,
        accessKeySecret: accessKeySecret,
        endpoint: endpoint,
        apiVersion: apiVersion,
    });

    const __request_options = {
        timeout: 3000, // default 3000 ms
        formatAction: true, // default true, format the action to Action
        formatParams: true, // default true, format the parameter name to first letter upper case
        method: 'GET', // set the http method, default is GET
        headers: {}, // set the http request headers
    };

    const CreateUploadVideo = function (pParameter) {
        if (!pParameter) {
            pParameter = {};
        }

        const defaultParameter = {};
        defaultParameter.Title = ``;
        defaultParameter.FileName = ``;
        defaultParameter.FileSize = ``;
        defaultParameter.Description = ``;
        defaultParameter.CoverURL = ``;
        defaultParameter.CateId = ``;
        defaultParameter.Tags = ``;
        defaultParameter.TemplateGroupId = ``;
        defaultParameter.UserData = JSON.stringify({});
        defaultParameter.StorageLocation = ``;

        const parameter = extend({}, defaultParameter, pParameter);

        const Title = parameter.Title;
        const FileName = parameter.FileName;
        const FileSize = parameter.FileSize;
        const Description = parameter.Description;
        const CoverURL = parameter.CoverURL;
        const CateId = parameter.CateId;
        const Tags = parameter.Tags;
        const TemplateGroupId = parameter.TemplateGroupId;
        const UserData = parameter.UserData;
        const StorageLocation = parameter.StorageLocation;

        const action = `CreateUploadVideo`;
        const params = {
            Title: Title,
            FileName: FileName,
            FileSize: FileSize,
            Description: Description,
            CoverURL: CoverURL,
            CateId: CateId,
            Tags: Tags,
            TemplateGroupId: TemplateGroupId,
            UserData: UserData,
            StorageLocation: StorageLocation,
        };
        return __client.request(action, params, __request_options);
    };

    const DeleteVideo = function (pParameter) {
        if (!pParameter) {
            pParameter = {};
        }

        const defaultParameter = {};
        defaultParameter.VideoIds = ``;

        const parameter = extend({}, defaultParameter, pParameter);

        const VideoIds = parameter.VideoIds;

        const action = `DeleteVideo`;
        const params = {
            VideoIds: VideoIds,
        };
        return __client.request(action, params, __request_options);
    };

    const GetVideoInfo = function (pParameter) {
        if (!pParameter) {
            pParameter = {};
        }

        const VideoId = pParameter.VideoId;

        const action = `GetVideoInfo`;
        const params = {
            VideoId: VideoId
        };
        return __client.request(action, params, __request_options);
    };

    const GetVideoInfos = function (pParameter) {
        if (!pParameter) {
            pParameter = {};
        }

        const VideoIds = pParameter.VideoIds;

        const action = `GetVideoInfos`;
        const params = {
            VideoIds: VideoIds
        };
        return __client.request(action, params, __request_options);
    };

    const GetVideoPlayAuth = function (pParameter) {
        if (!pParameter) {
            pParameter = {};
        }

        const VideoId = pParameter.VideoId;

        const action = `GetVideoPlayAuth`;
        const params = {
            VideoId: VideoId
        };
        return __client.request(action, params, __request_options);
    };

    const RefreshUploadVideo = function (pParameter) {
        if (!pParameter) {
            pParameter = {};
        }

        const VideoId = pParameter.VideoId;

        const action = `RefreshUploadVideo`;
        const params = {
            VideoId: VideoId
        };
        return __client.request(action, params, __request_options);
    };

    const SearchMedia = function (pParameter) {
        if (!pParameter) {
            pParameter = {};
        }

        const SearchType = pParameter.SearchType;
        const Fields = pParameter.Fields || `VideoId,CateId,Title`;
        const Match = pParameter.Match;
        const ScrollToken = pParameter.ScrollToken;
        const SortBy = pParameter.SortBy;
        const PageNo = pParameter.PageNo || 1;
        const PageSize = pParameter.PageSize || 100;

        const action = `SearchMedia`;
        const params = {
            SearchType: SearchType,
            Fields: Fields,
            Match: Match,
            ScrollToken: ScrollToken,
            SortBy: SortBy,
            PageNo: PageNo,
            PageSize: PageSize,
        };
        return __client.request(action, params, __request_options);
    };

    /**
     * CreateUploadVideo
     * <br>官方文档：https://help.aliyun.com/document_detail/55407.html?spm=a2c4g.11186623.2.22.5e8b6f93FU4IdJ
     *
     * @method CreateUploadVideo
     *
     * @param {object} pParameter
     * @param {string} pParameter.Title
     * @param {string} pParameter.FileName
     * @param {string} pParameter.CateId
     * @param {string} pParameter.TemplateGroupId
     * @param {string} pParameter.StorageLocation
     * @return {object} promise
     *
     * @demo src/system-docking/aliyun/vod/backend/client/demo/CreateUploadVideo.demo.js
     */
    __this.CreateUploadVideo = CreateUploadVideo;

    /**
     * DeleteVideo
     * <br>官方文档：https://help.aliyun.com/document_detail/52837.html?spm=a2c4g.11186623.2.32.5e8b6f93FU4IdJ
     *
     * @method DeleteVideo
     *
     * @param {object} pParameter
     * @return {object} promise
     *
     * @demo src/system-docking/aliyun/vod/backend/client/demo/DeleteVideo.demo.js
     */
    __this.DeleteVideo = DeleteVideo;

    /**
     * GetVideoInfo
     * <br>官方文档：https://help.aliyun.com/document_detail/52835.html?spm=a2c4g.11186623.2.30.5e8b6f93FU4IdJ
     *
     * @method GetVideoInfo
     *
     * @param {object} pParameter
     * @param {string} pParameter.VideoId
     * @return {object} promise
     *
     * @demo src/system-docking/aliyun/vod/backend/client/demo/GetVideoInfo.demo.js
     */
    __this.GetVideoInfo = GetVideoInfo;

    /**
     * GetVideoInfos
     * <br>官方文档：https://help.aliyun.com/document_detail/86042.html?spm=a2c4g.11186623.2.36.5e8b6f93JY6nQr
     *
     * @method GetVideoInfos
     *
     * @param {object} pParameter
     * @param {string} pParameter.VideoIds
     * @return {object} promise
     *
     * @demo src/system-docking/aliyun/vod/backend/client/demo/GetVideoInfos.demo.js
     */
    __this.GetVideoInfos = GetVideoInfos;

    /**
     * GetVideoPlayAuth
     * <br>官方文档：https://help.aliyun.com/document_detail/52833.html?spm=a2c4g.11186623.6.669.6c8d53c0azLz5z
     *
     * @method GetVideoPlayAuth
     *
     * @param {object} pParameter
     * @param {string} pParameter.VideoId
     * @return {object} promise
     *
     * @demo src/system-docking/aliyun/vod/backend/client/demo/GetVideoPlayAuth.demo.js
     */
    __this.GetVideoPlayAuth = GetVideoPlayAuth;

    /**
     * RefreshUploadVideo
     * <br>官方文档：https://help.aliyun.com/document_detail/55408.html?spm=a2c4g.11186623.2.23.5e8b6f93OpDvwh
     *
     * @method RefreshUploadVideo
     *
     * @param {object} pParameter
     * @param {string} pParameter.VideoId
     * @return {object} promise
     *
     * @demo src/system-docking/aliyun/vod/backend/client/demo/RefreshUploadVideo.demo.js
     */
    __this.RefreshUploadVideo = RefreshUploadVideo;

    /**
     * SearchMedia
     * <br>官方文档：https://help.aliyun.com/document_detail/86044.html?spm=a2c4g.11186623.6.672.72fc17e39nXMQK
     *
     * @method SearchMedia
     *
     * @param {object} pParameter
     * @return {object} promise
     *
     * @demo src/system-docking/aliyun/vod/backend/client/demo/SearchMedia.demo.js
     */
    __this.SearchMedia = SearchMedia;

};