//<editor-fold desc="node core modules">
const path = require('path');
//</editor-fold>

//<editor-fold desc="npm modules">
const fs = require('fs-extra');
const extend = require('extend');
const Q = require('q');
const _ = require('underscore');
const moment = require('moment');
const URI = require('urijs');
const rp = require('request-promise');
//</editor-fold>

const join = require(`../../../../uri/join`);

/**
 * NL类库
 *
 * @module NL
 */

/**
 * goeasy客户端（服务端版本）
 * <div>
 * <span class="tag margin-right">不支持浏览器环境</span>
 * <span class="tag margin-right">支持node环境</span>
 * </div>
 *
 * @class NL.systemDocking.goeasy.backend.client
 * @param {object} pInitParameter
 * @param {string} pInitParameter.appKey goeasy的AppKey（从goeasy后台查看）
 * @param {string} pInitParameter.origin goeasy的REST origin（从goeasy后台查看）
 * @constructor
 * @return {object} NL.systemDocking.goeasy.backend.client实例
 *
 * @demo src/system-docking/goeasy/backend/client/demo/default.demo.js
 */
const client = function (pInitParameter) {
    const __this = this;

    const defaultInitParameter = {};
    defaultInitParameter.appKey = '';
    defaultInitParameter.origin = '';

    const parameter = extend(true, {}, defaultInitParameter, pInitParameter);

    const appKey = parameter.appKey;
    if (!appKey) {
        const msg = 'client parameter error, code=001';
        console.error(msg);
        return Q.reject({msg: msg});
    }

    const origin = parameter.origin;
    if (!origin) {
        const msg = 'client parameter error, code=002';
        console.error(msg);
        return Q.reject({msg: msg});
    }

    const url_publish = join(origin, `publish`);
    // console.log(`url_publish=${url_publish}`);

    /**
     * 发送消息
     *
     * @method publish
     * @param {object} pParameter
     * @param {string} pParameter.channel 频道
     * @param {string} pParameter.content 消息
     * @return {object} 发送消息成功的promise
     *
     */
    const publish = function (pParameter) {
        if (!pParameter) {
            pParameter = {};
        }

        const channel = pParameter.channel;
        const content = pParameter.content;

        if (!channel) {
            const msg = 'publish parameter error, code=001';
            console.error(msg);
            return Q.reject({msg: msg});
        }
        if (!content) {
            const msg = 'publish parameter error, code=002';
            console.error(msg);
            return Q.reject({msg: msg});
        }

        return Q.when()
            .then(function () {
                const options = {
                    method: `post`,
                    uri: url_publish,
                    form: {
                        appkey: appKey,
                        channel: channel,
                        content: content,
                    },
                    json: true,
                    // proxy: `http://127.0.0.1:8888`,
                };

                return rp(options);
            })
            .catch(function (ex) {
                console.error(ex);
                if (ex) {
                    if (ex.stack) {
                        console.error(ex.stack);
                    }
                }
                return Q.reject(ex);
            })
            ;
    };

    __this.publish = publish;

};

module.exports = client;