const Q = require('q');
const _ = require('underscore');

const join = require(`../../../uri/join`);
const request_webApiType_v3 = require(`../../../model/request_webApiType_v3`);

const dict_app_environment_join = require(`./dicts/dict_app_environment_join`);

const __request = function (pParameter) {
    if (!pParameter) {
        pParameter = {};
    }

    const environment = pParameter.environment;
    if (!environment) {
        const msg = '__request parameter error, code=001';
        console.error(msg);
        return Q.reject({msg: msg});
    }

    const url_suffix = pParameter.url_suffix;
    if (!url_suffix) {
        const msg = '__request parameter error, code=002';
        console.error(msg);
        return Q.reject({msg: msg});
    }

    const get_token = pParameter.get_token;

    const data = pParameter.data;

    const app_key = `default-qiyehao`;

    return Q.when()
        .then(function () {
            return request_webApiType_v3({
                url_suffix: url_suffix,
                data: data,
                get_url: function () {
                    const list = dict_app_environment_join.entityList;
                    const query = _.chain(list)
                        .find(function (m) {
                            return 1 === 1
                                && m.appKey === app_key
                                && m.environment === environment;
                        })
                        .value();

                    if (!query) {
                        let msg = '';
                        msg += '__request parameter error, code=701';
                        msg += `|app_key=${app_key}`;
                        msg += `|url_suffix=${url_suffix}`;
                        console.error(msg);
                        return ``;
                    }

                    const url_prefix = query.urlPrefix;
                    const url = join(url_prefix, url_suffix);
                    return url;
                },
                get_token: get_token,
            });
        })
        .then(function (data) {
            if (!data) {
                const msg = '__request parameter error, code=301';
                console.error(msg);
                return Q.reject({msg: msg});
            }

            return Q.resolve(data);
        })
        .catch(function (ex) {
            console.error(ex);
            if (ex) {
                if (ex.stack) {
                    console.error(ex.stack);
                }
            }
            return Q.reject(ex);
        })
        ;
};

module.exports = __request;
