//<editor-fold desc="node core modules">
// const path = require('path');
//</editor-fold>

//<editor-fold desc="npm modules">
// const fs = require('fs-extra');
const extend = require('extend');
const Q = require('q');
const _ = require('underscore');
// const moment = require('moment');
// const URI = require('urijs');
//</editor-fold>

/**
 * NL类库
 *
 * @module nllib
 */

/**
 * 获取企业号token
 *
 * <div>
 * <span class="tag margin-right">不支持浏览器环境</span>
 * <span class="tag margin-right">支持node环境</span>
 * </div>
 * 
 * @class nllib/src/system-docking/qyh/backend/get_token
 * @static
 *
 * @param {object} pParameter
 *
 * @param {string} pParameter.environment 环境
 * @param {string} pParameter.userName 用户名称
 * @param {string} pParameter.password 用户密码
 *
 * @return {object} promise
 * <br>promise.then(data => {});
 * <br>data.token = ``;//token
 *
 * @demo src/system-docking/qyh/backend/demo/get_token.demo.js
 *
 */

const webBase_api_sso_loginBySecurityAsync = require(`./webBase_api_sso_loginBySecurityAsync.js`);

const get_token = function (pParameter) {
    if (!pParameter) {
        pParameter = {};
    }

    const environment = pParameter.environment;
    const userName = pParameter.userName;
    const password = pParameter.password;

    return Q.when()
        .then(function () {
            return webBase_api_sso_loginBySecurityAsync({
                environment: environment,
                userName: userName,
                password: password,
            });
        })
        .then(function (data) {
            const d = {};
            d.token = data.id;
            return Q.resolve(d);
        })
        .catch(function (ex) {
            console.error(ex);
            if (ex) {
                if (ex.stack) {
                    console.error(ex.stack);
                }
            }
            return Q.reject(ex);
        })
        ;
};

module.exports = get_token;
