const Q = require('q');

const get_config = require(`./get_config`);
const __request = require(`./__request`);


const webBase_api_sso_loginBySecurityAsync = function (pParameter) {
    if (!pParameter) {
        pParameter = {};
    }

    // 接口文档：
    // http://vote.jxt189.com:9368/swagger/api-docs/#/Sso/Sso_LoginBySecurityAsync
    // pParameter.userName = ``;
    // pParameter.password = ``;
    // pParameter.expireMin = 0;

    const environment = pParameter.environment;
    if (!environment) {
        const msg = 'webBase_api_sso_loginBySecurityAsync parameter error, code=001';
        console.error(msg);
        return Q.reject({msg: msg});
    }

    const config = get_config({environment: environment});

    const userName = pParameter.userName || config.user.userName;
    const password = pParameter.password || config.user.password;
    const expireMin = pParameter.expireMin || 0;

    return Q.when()
        .then(function () {
            return __request({
                environment: environment,
                url_suffix: `webBase/api/sso/loginBySecurityAsync`,
                data: {
                    userName: userName,
                    password: password,
                    expireMin: expireMin,
                },
                get_token: function () {
                    return ``;
                }
            });
        })
        .then(function (data) {
            if (!data) {
                const msg = 'webBase_api_sso_loginBySecurityAsync parameter error, code=501';
                console.error(msg);
                return Q.reject({msg: msg});
            }

            return Q.resolve(data);
        })
        .catch(function (ex) {
            console.error(ex);
            if (ex) {
                if (ex.stack) {
                    console.error(ex.stack);
                }
            }
            return Q.reject(ex);
        })
        ;
};

module.exports = webBase_api_sso_loginBySecurityAsync;
