const Q = require('q');
const _ = require('underscore');

const join = require(`../../../uri/join`);
const request_webApiType_v2_1 = require(`../../../model/request_webApiType_v2_1`);

const dict_app_environment_join = require(`./dicts/dict_app_environment_join`);

const __request = function (pParameter) {
    if (!pParameter) {
        pParameter = {};
    }

    const get_environment = pParameter.get_environment;
    const get_token = pParameter.get_token;

    const url_suffix = pParameter.url_suffix;
    const data = pParameter.data;

    return Q.when()
        .then(function () {
            return request_webApiType_v2_1({
                app_key: `default-qiyehao`,
                url_suffix: url_suffix,
                data: data,
                get_environment: get_environment,
                get_token: get_token,
                get_url: function (args) {
                    const __parameter = args.parameter;
                    const app_key = __parameter.app_key;
                    const url_suffix = __parameter.url_suffix;
                    const environment = __parameter.get_environment();

                    const list = dict_app_environment_join.entityList;
                    const query = _.chain(list)
                        .find(function (m) {
                            return 1 === 1
                                && m.appKey === app_key
                                && m.environment === environment;
                        })
                        .value();

                    if (!query) {
                        let msg = 'request_webApiType_v2_1 parameter error, code=701';
                        msg += `|app_key=${app_key}`;
                        msg += `|url_suffix=${url_suffix}`;
                        console.error(msg);
                        return Q.reject({msg: msg});
                    }

                    const url_prefix = query.urlPrefix;
                    const url = join(url_prefix, url_suffix);
                    return url;
                },
            });
        })
        .then(function (data) {
            if (!data) {
                const msg = '__request parameter error, code=001';
                console.error(msg);
                return Q.reject({msg: msg});
            }

            const list = data.list;
            if (!list) {
                const msg = '__request parameter error, code=002';
                console.error(msg);
                return Q.reject({msg: msg});
            }
            if (list.length <= 0) {
                const msg = '__request parameter error, code=003';
                console.error(msg);
                return Q.reject({msg: msg});
            }

            return Q.resolve(data);
        })
        .catch(function (ex) {
            console.error(ex);
            if (ex) {
                if (ex.stack) {
                    console.error(ex.stack);
                }
            }
            return Q.reject(ex);
        })
        ;
};

module.exports = __request;
