const Q = require('q');
const _ = require('underscore');
const axios = require('axios');
const join = require(`../../../uri/join`);

const dict_app_environment_join = require(`./dicts/dict_app_environment_join`);

const webBase_api_sso_loginBySecurityAsync = function (pParameter) {
    if (!pParameter) {
        pParameter = {};
    }

    const environment = pParameter.environment;
    const userName = pParameter.userName;
    const password = pParameter.password;

    if (!environment) {
        const msg = 'webBase_api_sso_loginBySecurityAsync parameter error, code=001';
        console.error(msg);
        return Q.reject({msg: msg});
    }
    if (!userName) {
        const msg = 'webBase_api_sso_loginBySecurityAsync parameter error, code=002';
        console.error(msg);
        return Q.reject({msg: msg});
    }
    if (!password) {
        const msg = 'webBase_api_sso_loginBySecurityAsync parameter error, code=003';
        console.error(msg);
        return Q.reject({msg: msg});
    }

    const get_url_prefix = function () {
        const query = _.chain(dict_app_environment_join.entityList)
            .find(function (m) {
                return 1 === 1
                    && m.environment === environment
                    && m.appKey === `default-qiyehao`
                    ;
            })
            .value()
        ;
        if (!query) {
            const msg = 'get_url_prefix parameter error, code=001';
            console.error(msg);
            return Q.reject({msg: msg});
        }

        const url_prefix = query.urlPrefix;
        if (!url_prefix) {
            const msg = 'get_url_prefix parameter error, code=002';
            console.error(msg);
            return Q.reject({msg: msg});
        }

        return url_prefix;
    };
    const url_prefix = get_url_prefix();
    if (!url_prefix) {
        const msg = 'webBase_api_sso_loginBySecurityAsync parameter error, code=004';
        console.error(msg);
        return Q.reject({msg: msg});
    }

    const url = join(url_prefix, `webbase/api/sso/loginBySecurityAsync`);

    return Q.when()
        .then(function () {
            return axios({
                method: 'post',
                url: url,
                data: {
                    userName: userName,
                    password: password,
                }
            });
        })
        .then(function (response) {
            const data = response.data;
            // console.log(data);
            if (!data) {
                const msg = 'webBase_api_sso_loginBySecurityAsync parameter error, code=100';
                console.error(msg);
                return Q.reject({msg: msg});
            }

            const id = data.id;
            if (!id) {
                const msg = 'webBase_api_sso_loginBySecurityAsync parameter error, code=101';
                console.error(msg);
                return Q.reject({msg: msg});
            }

            const list = data.list;
            if (!list) {
                const msg = 'webBase_api_sso_loginBySecurityAsync parameter error, code=102';
                console.error(msg);
                return Q.reject({msg: msg});
            }

            let first_user_id = ``;
            if (list.length > 0) {
                const first_001 = list[0];
                first_user_id = first_001.userId;
            }
            if (!first_user_id) {
                const msg = 'webBase_api_sso_loginBySecurityAsync parameter error, code=103';
                console.error(msg);
                return Q.reject({msg: msg});
            }

            data.token = id;
            data.first_user_id = first_user_id;

            return Q.resolve(data);
        })
        .catch(function (ex) {
            console.error(ex);
            if (ex) {
                if (ex.stack) {
                    console.error(ex.stack);
                }
            }
            return Q.reject(ex);
        })
        ;

};

module.exports = webBase_api_sso_loginBySecurityAsync;
