const Q = require('q');

const __request = require(`./__request`);

const webBase_api_v2_user_getUserClassList = function (pParameter) {
    if (!pParameter) {
        pParameter = {};
    }

    return Q.when()
        .then(function () {
            return __request({
                url_suffix: `webBase/api/v2/user/getUserClassList`,
                data: {
                    userId: ``,
                    platformKey: ``,
                },
                get_environment: pParameter.get_environment,
                get_token: pParameter.get_token,
            });
        })
        .then(function (data) {
            if (!data) {
                const msg = 'webBase_api_v2_user_getUserClassList parameter error, code=001';
                console.error(msg);
                return Q.reject({msg: msg});
            }

            const list = data.list;
            if (!list) {
                const msg = 'webBase_api_v2_user_getUserClassList parameter error, code=002';
                console.error(msg);
                return Q.reject({msg: msg});
            }

            let first_class_id = ``;
            let first_course_id = ``;
            let first_db_id = ``;

            if (list.length > 0) {
                const first_001 = list[0];
                first_course_id = first_001.courseId;
                first_db_id = first_001.dbId;

                const classList = first_001.classList;
                if (classList) {
                    if (classList.length > 0) {
                        const first_002 = classList[0];
                        first_class_id = first_002.classId;
                    }
                }
            }

            if (!first_class_id) {
                const msg = 'webBase_api_v2_user_getUserClassList parameter error, code=003';
                console.error(msg);
                return Q.reject({msg: msg});
            }
            if (!first_course_id) {
                const msg = 'webBase_api_v2_user_getUserClassList parameter error, code=004';
                console.error(msg);
                return Q.reject({msg: msg});
            }
            if (!first_db_id) {
                const msg = 'webBase_api_v2_user_getUserClassList parameter error, code=005';
                console.error(msg);
                return Q.reject({msg: msg});
            }

            data.first_class_id = first_class_id;
            data.first_course_id = first_course_id;
            data.first_db_id = first_db_id;

            return Q.resolve(data);
        })
        .catch(function (ex) {
            console.error(ex);
            if (ex) {
                if (ex.stack) {
                    console.error(ex.stack);
                }
            }
            return Q.reject(ex);
        })
        ;
};

module.exports = webBase_api_v2_user_getUserClassList;


// 好像当使用es6的某些特性时，必须要这样写（但是运行还是报错：TypeError: webBase_api_v2_user_getUserClassList is not a function）
// export default webBase_api_v2_user_getUserClassList;
// 不能这样写
// module.exports = webBase_api_v2_user_getUserClassList;
// 否则会在运行时报错：Cannot assign to read only property 'exports' of object '#<Object>'

// 但是，当使用export default webBase_api_v2_user_getUserClassList;时，该模块又不能直接在node环境下运行

// 暂时未找到以下方案：
// 方案一：
// 1）源文件使用CommonJs模块规范来写，即使用require + module.exports
// 2）源文件使用了es6的特性，例如async
// 3）生成一个可以在普通浏览器运行的js，运行时不报【Cannot assign to read only property 'exports' of object '#<Object>'】
// 4）该源文件可以直接在node环境运行，不报错
// 方案二：
// 1）源文件使用es模块规范来写，即使用import + export
// 2）源文件使用了es6的特性，例如async
// 3）生成一个可以在普通浏览器运行的js，运行时不报【Cannot assign to read only property 'exports' of object '#<Object>'】
// 4）该源文件可以直接在node环境运行，不报错

// 最后换了一种想法：
// 出现该问题是因为使用了babel，
// babel实际是将es6的语法变成es5的语法，
// 所以，如果浏览器本身就是支持es6语法的话，根本不需要使用babel去做转换
// 而现在许多主流的浏览器，已经支持常用的es6语法特性（例如async），
// 因此，最后决定：不使用babel（babel-loader）做转换
