//<editor-fold desc="node core modules">
// const path = require('path');
//</editor-fold>

//<editor-fold desc="npm modules">
// const fs = require('fs-extra');
const extend = require('extend');
const Q = require('q');
const _ = require('underscore');
// const moment = require('moment');
// const URI = require('urijs');
//</editor-fold>

const __v1_sso_loginBySecurity = require(`./__v1_sso_loginBySecurity`);
const __v1_sso_confirmBySecurity = require(`./__v1_sso_confirmBySecurity`);
const __v2_oauth_token_get = require(`./__v2_oauth_token_get`);

/**
 * NL类库
 *
 * @module nllib
 */

/**
 * 获取sso token
 *
 * <div>
 * <span class="tag margin-right">不支持浏览器环境</span>
 * <span class="tag margin-right">支持node环境</span>
 * </div>
 *
 * @class nllib/src/system-docking/sso/backend/get_token
 * @static
 *
 * @param {object} pParameter
 *
 * @param {string} pParameter.environment 环境
 * @param {string} pParameter.userName 用户名称
 * @param {string} pParameter.password 用户密码
 *
 * @return {object} promise
 * <br>promise.then(data => {});
 * <br>data.token = ``;//token
 *
 * @demo src/system-docking/sso/backend/demo/get_token.demo.js
 *
 */

const get_token = function (pParameter) {
    const defaultParameter = {};
    defaultParameter.environment = '';

    const parameter = extend(true, {}, defaultParameter, pParameter);

    const environment = parameter.environment;
    const userName = parameter.userName;
    const password = parameter.password;
    const platformKey = parameter.platformKey;

    return Q.when()
        .then(function () {
            return __v1_sso_loginBySecurity({
                environment: environment,
                userName: userName,
                password: password,
            });
        })
        .then(function (data) {
            const list = data.data;
            if (list.length === 1) {
                return Q.resolve(data);
            }

            const token = data.token;

            const query = _.chain(list)
                .find(function (m) {
                    return m.platformKey === platformKey;
                })
                .value()
            ;
            if (!query) {
                const msg = 'get_token parameter error, code=001';
                console.error(msg);
                return Q.reject({msg: msg});
            }

            const userId = query.userId;

            return __v1_sso_confirmBySecurity({
                environment: environment,
                userId: userId,
                platformKey: platformKey,
                token: token,
            });

        })
        .then(function (data) {
            const ticket = data.token;

            return __v2_oauth_token_get({
                environment: environment,
                ticket: ticket,
            });
        })
        .catch(function (ex) {
            console.error(ex);
            if (ex) {
                if (ex.stack) {
                    console.error(ex.stack);
                }
            }
            return Q.reject(ex);
        })
        ;
};
module.exports = get_token;