//<editor-fold desc="node core modules">
// const path = require(`path`);
//</editor-fold>

//<editor-fold desc="npm modules">
// const fs = require(`fs-extra`);
const extend = require(`extend`);
const $ = require(`jquery`);
const Q = require(`q`);
const _ = require(`underscore`);
// const moment = require(`moment`);
// const URI = require(`urijs`);
//</editor-fold>

const __get_sso_config = require(`./__get_sso_config`);
const join = require(`../../../uri/join`);

module.exports = function (pParameter) {
    const defaultParameter = {};
    defaultParameter.environment = ``;

    const parameter = extend(true, {}, defaultParameter, pParameter);

    const environment = parameter.environment;

    const sso_config = __get_sso_config({environment: environment});
    // console.log(`sso_config=${JSON.stringify(sso_config, null, 4)}`);

    const userId = parameter.userId;
    const selectToken = parameter.selectToken;
    const platformKey = parameter.platformKey;

    const url = join(sso_config.uriPrefix, `v1/sso/selectUser.jsonp`);

    const options = {
        type: `get`,
        contentType: `application/json;charset=utf-8`,
        dataType: `jsonp`,
        url: url,
        data: {
            userid: userId,
            selectToken: selectToken,
            platformKey: platformKey,
        },
    };

    return Q.when()
        .then(function () {
            return $.ajax(options);
        })
        .then(function (data) {
            if (!data) {
                const msg = '__v1_sso_selectUser_jsonp parameter error, code=000';
                console.error(msg);
                return Q.reject({msg: msg});
            }

            const status = data.status;
            if (!_.contains([`success`, `multiple`], status)) {
                const msg = '__v1_sso_selectUser_jsonp parameter error, code=001';
                console.error(msg);
                return Q.reject({msg: msg});
            }

            if (status === `success`) {
                const NLTicket = data.NLTicket;
                if (!NLTicket) {
                    const msg = '__v1_sso_selectUser_jsonp parameter error, code=101';
                    console.error(msg);
                    return Q.reject({msg: msg});
                }

                const platformKey = data.platformKey;
                if (!platformKey) {
                    const msg = '__v1_sso_selectUser_jsonp parameter error, code=102';
                    console.error(msg);
                    return Q.reject({msg: msg});
                }
            }

            if (status === `multiple`) {
                const selectToken = data.selectToken;
                if (!selectToken) {
                    const msg = '__v1_sso_selectUser_jsonp parameter error, code=201';
                    console.error(msg);
                    return Q.reject({msg: msg});
                }

                const users = data.users;
                if (!users) {
                    const msg = '__v1_sso_selectUser_jsonp parameter error, code=202';
                    console.error(msg);
                    return Q.reject({msg: msg});
                }

                if (users.length <= 1) {
                    const msg = '__v1_sso_selectUser_jsonp parameter error, code=203';
                    console.error(msg);
                    return Q.reject({msg: msg});
                }
            }

            return Q.resolve(data);
        })
        .catch(function (ex) {
            console.error(ex);
            if (ex) {
                if (ex.stack) {
                    console.error(ex.stack);
                }
            }
            return Q.reject(ex);
        })
        ;
};