describe('redictType=101，token和appKey为必填参数', function () {

    let testCaseList = [
        {
            msg: '默认使用方法，只传入token和appKey参数',
            src: {
                url: 'http://127.0.0.1/index-redirect.html?appkey=demo&token=RT-ST-ae9602e4eff84dceaf7d670a9a55dd6d'
            },
            dest: {
                "flag": true,
                "msg": "",
                "data": {
                    "token": "RT-ST-ae9602e4eff84dceaf7d670a9a55dd6d",
                    "appKey": "demo",
                    "redirectUrl": "../demo/index-demo.html?appkey=demo"
                }
            }
        },

        {
            msg: '会将传入的redirectType和token参数删除，appKey参数不删除',
            src: {
                url: 'http://127.0.0.1/index-redirect.html?redirecttype=101&appkey=demo&token=RT-ST-ae9602e4eff84dceaf7d670a9a55dd6d'
            },
            dest: {
                "flag": true,
                "msg": "",
                "data": {
                    "token": "RT-ST-ae9602e4eff84dceaf7d670a9a55dd6d",
                    "appKey": "demo",
                    "redirectUrl": "../demo/index-demo.html?appkey=demo"
                }
            }
        },

        {
            msg: '会将除redirectType和token参数外的所有queryString参数，追加到新的重定向地址中',
            src: {
                url: 'http://127.0.0.1/index-redirect.html?redirecttype=101&appkey=demo&Hello=JavaScript&WORLD=VeryGood&token=RT-ST-ae9602e4eff84dceaf7d670a9a55dd6d'
            },
            dest: {
                "flag": true,
                "msg": "",
                "data": {
                    "token": "RT-ST-ae9602e4eff84dceaf7d670a9a55dd6d",
                    "appKey": "demo",
                    "redirectUrl": "../demo/index-demo.html?appkey=demo&Hello=JavaScript&WORLD=VeryGood"
                },
            }
        },
    ];

    _.each(testCaseList, function (testCase) {
        const src = testCase.src;
        const dest = testCase.dest;

        it(testCase.msg, function (done) {
            const aRedirectHelper = new NL.systemDocking.sso.frontend.redirectHelper({
                redirectType: '101',
                url: src.url,
            });

            aRedirectHelper.init();
            aRedirectHelper.getReturnValue().then(function (aReturnValue) {
                console.log(aReturnValue);
                expect(aReturnValue.flag).to.eql(dest.flag);
                expect(aReturnValue.msg).to.eql(dest.msg);

                expect(aReturnValue.data.appKey).to.eql(dest.data.appKey);
                expect(aReturnValue.data.token).to.eql(dest.data.token);

                const uri_expect = URI(dest.data.redirectUrl);
                const uri_actual = URI(aReturnValue.data.redirectUrl);

                expect(uri_actual.path()).to.eql(uri_expect.path());

                const queryStringObject_expect = NL.uri.get_query_string_object({uri: dest.data.redirectUrl});
                const queryStringObject_actual = NL.uri.get_query_string_object({uri: aReturnValue.data.redirectUrl});

                expect(queryStringObject_actual.appkey).to.eql(queryStringObject_expect.appkey);
                expect(queryStringObject_actual).to.have.property('nlVersion');
                done();
            }).catch(function (ex) {
                console.error(ex);
            });
        });
    })


});