describe('redictType=110，ticket或者token、environment和redirectUrl为必填参数', function () {

    describe('ticket与token只存在token', function () {

        const testCaseList = [
            {
                msg: '默认使用方法，只传入token、environment和redirectUrl参数',
                src: {
                    url: `http://127.0.0.1/index-redirect.html?environment=alpha&token=RT-ST-ae9602e4eff84dceaf7d670a9a55dd6d&redirectUrl=` + encodeURIComponent('http://127.0.0.1/index-app.html'),
                },
                dest: {
                    "flag": true,
                    "msg": "",
                    "data": {
                        "environment": "alpha",
                        "ticket": "",
                        "token": "RT-ST-ae9602e4eff84dceaf7d670a9a55dd6d",
                        "appKey": "",
                        "redirectUrl": "http://127.0.0.1/index-app.html",
                        "origin_redirectUrl_decode": "http://127.0.0.1/index-app.html",
                        "origin_redirectUrl_encode": encodeURIComponent('http://127.0.0.1/index-app.html')
                    }
                }
            },

            {
                msg: '会将传入的redirectType、token、environment和redirectUrl参数删除',
                src: {
                    url: `http://127.0.0.1/index-redirect.html?redirecttype=110&environment=alpha&token=RT-ST-ae9602e4eff84dceaf7d670a9a55dd6d&redirectUrl=` + encodeURIComponent('http://127.0.0.1/index-app.html'),
                },
                dest: {
                    "flag": true,
                    "msg": "",
                    "data": {
                        "environment": "alpha",
                        "ticket": "",
                        "token": "RT-ST-ae9602e4eff84dceaf7d670a9a55dd6d",
                        "appKey": "",
                        "redirectUrl": "http://127.0.0.1/index-app.html",
                        "origin_redirectUrl_decode": "http://127.0.0.1/index-app.html",
                        "origin_redirectUrl_encode": encodeURIComponent('http://127.0.0.1/index-app.html')
                    }
                }
            },

            {
                msg: '会将除redirectType、token、environment和redirectUrl参数外的所有queryString参数，追加到新的重定向地址中',
                src: {
                    url: `http://127.0.0.1/index-redirect.html?redirecttype=110&environment=alpha&token=RT-ST-ae9602e4eff84dceaf7d670a9a55dd6d&redirectUrl=` + encodeURIComponent('http://127.0.0.1/index-app.html') + '&Hello=JavaScript&WORLD=VeryGood',
                },
                dest: {
                    "flag": true,
                    "msg": "",
                    "data": {
                        "environment": "alpha",
                        "ticket": "",
                        "token": "RT-ST-ae9602e4eff84dceaf7d670a9a55dd6d",
                        "appKey": "",
                        "redirectUrl": "http://127.0.0.1/index-app.html?Hello=JavaScript&WORLD=VeryGood",
                        "origin_redirectUrl_decode": "http://127.0.0.1/index-app.html",
                        "origin_redirectUrl_encode": encodeURIComponent('http://127.0.0.1/index-app.html')
                    }
                }
            },
        ];

        _.each(testCaseList, function (testCase) {
            const src = testCase.src;
            const dest = testCase.dest;

            it(testCase.msg, function (done) {
                const aRedirectHelper = new NL.systemDocking.sso.frontend.redirectHelper({
                    redirectType: '110',
                    url: src.url,
                });

                aRedirectHelper.init();
                aRedirectHelper.getReturnValue().then(function (aReturnValue) {
                    console.log(aReturnValue);
                    expect(aReturnValue.flag).to.eql(dest.flag);
                    expect(aReturnValue.msg).to.eql(dest.msg);

                    expect(aReturnValue.data.environment).to.eql(dest.data.environment);
                    expect(aReturnValue.data.ticket).to.not.be.ok();
                    expect(aReturnValue.data.token).to.eql(dest.data.token);

                    const uri_expect = URI(dest.data.redirectUrl);
                    const uri_actual = URI(aReturnValue.data.redirectUrl);

                    expect(uri_actual.path()).to.eql(uri_expect.path());

                    const queryStringObject_expect = NL.uri.get_query_string_object({uri: dest.data.redirectUrl});
                    const queryStringObject_actual = NL.uri.get_query_string_object({uri: aReturnValue.data.redirectUrl});

                    expect(queryStringObject_actual.appkey).to.eql(queryStringObject_expect.appkey);
                    expect(queryStringObject_actual).to.have.property('nlVersion');

                    done();
                }).catch(function (ex) {
                    console.error(ex);
                });
            });
        })

    });

    describe('ticket与token只存在ticket', function () {
        const testCaseList = [
            {
                msg: '默认使用方法，只传入ticket、environment和redirectUrl参数',
                src: {
                    url: `http://127.0.0.1/index-redirect.html?environment=alpha&ticket=RT-ST-ae9602e4eff84dceaf7d670a9a55dd6d&redirectUrl=` + encodeURIComponent('http://127.0.0.1/index-app.html'),
                },
                dest: {
                    "flag": true,
                    "msg": "",
                    "data": {
                        "environment": "alpha",
                        "ticket": "RT-ST-ae9602e4eff84dceaf7d670a9a55dd6d",
                        "token": "999",
                        "appKey": "",
                        "redirectUrl": "http://127.0.0.1/index-app.html",
                        "origin_redirectUrl_decode": "http://127.0.0.1/index-app.html",
                        "origin_redirectUrl_encode": encodeURIComponent('http://127.0.0.1/index-app.html')
                    }
                }
            },

            {
                msg: '会将传入的redirectType、ticket、environment和redirectUrl参数删除',
                src: {
                    url: `http://127.0.0.1/index-redirect.html?redirecttype=110&environment=alpha&ticket=RT-ST-ae9602e4eff84dceaf7d670a9a55dd6d&redirectUrl=` + encodeURIComponent('http://127.0.0.1/index-app.html'),
                },
                dest: {
                    "flag": true,
                    "msg": "",
                    "data": {
                        "environment": "alpha",
                        "ticket": "RT-ST-ae9602e4eff84dceaf7d670a9a55dd6d",
                        "token": "999",
                        "appKey": "",
                        "redirectUrl": "http://127.0.0.1/index-app.html",
                        "origin_redirectUrl_decode": "http://127.0.0.1/index-app.html",
                        "origin_redirectUrl_encode": encodeURIComponent('http://127.0.0.1/index-app.html')
                    }
                }
            },

            {
                msg: '会将除redirectType、ticket、environment和redirectUrl参数外的所有queryString参数，追加到新的重定向地址中',
                src: {
                    url: `http://127.0.0.1/index-redirect.html?redirecttype=110&environment=alpha&ticket=RT-ST-ae9602e4eff84dceaf7d670a9a55dd6d&redirectUrl=` + encodeURIComponent('http://127.0.0.1/index-app.html') + '&Hello=JavaScript&WORLD=VeryGood',
                },
                dest: {
                    "flag": true,
                    "msg": "",
                    "data": {
                        "environment": "alpha",
                        "ticket": "RT-ST-ae9602e4eff84dceaf7d670a9a55dd6d",
                        "token": "999",
                        "appKey": "",
                        "redirectUrl": "http://127.0.0.1/index-app.html?Hello=JavaScript&WORLD=VeryGood",
                        "origin_redirectUrl_decode": "http://127.0.0.1/index-app.html",
                        "origin_redirectUrl_encode": encodeURIComponent('http://127.0.0.1/index-app.html')
                    }
                }
            },
        ];

        _.each(testCaseList, function (testCase) {
            const src = testCase.src;
            const dest = testCase.dest;

            it(testCase.msg, function (done) {
                const aRedirectHelper = new NL.systemDocking.sso.frontend.redirectHelper({
                    redirectType: '110',
                    url: src.url,
                    redirectTypeParameter: {
                        '110': {
                            get_token: function (args) {
                                const ticket = args.ticket;
                                const data = {};
                                data.token = `999`;
                                return Q.resolve(data);
                            }
                        }
                    }
                });

                aRedirectHelper.init();
                aRedirectHelper.getReturnValue().then(function (aReturnValue) {
                    console.log(aReturnValue);
                    expect(aReturnValue.flag).to.eql(dest.flag);
                    expect(aReturnValue.msg).to.eql(dest.msg);

                    expect(aReturnValue.data.environment).to.eql(dest.data.environment);
                    expect(aReturnValue.data.ticket).to.eql(dest.data.ticket);
                    expect(aReturnValue.data.token).to.eql(dest.data.token);

                    const uri_expect = URI(dest.data.redirectUrl);
                    const uri_actual = URI(aReturnValue.data.redirectUrl);

                    expect(uri_actual.path()).to.eql(uri_expect.path());

                    const queryStringObject_expect = NL.uri.get_query_string_object({uri: dest.data.redirectUrl});
                    const queryStringObject_actual = NL.uri.get_query_string_object({uri: aReturnValue.data.redirectUrl});

                    expect(queryStringObject_actual.appkey).to.eql(queryStringObject_expect.appkey);
                    expect(queryStringObject_actual).to.have.property('nlVersion');

                    done();
                }).catch(function (ex) {
                    console.error(ex);
                });
            });
        })
    });

    describe('ticket与token同时存在（都有值），以ticket为准', function () {
        const testCaseList = [
            {
                msg: '默认使用方法，只传入ticket、environment和redirectUrl参数',
                src: {
                    url: `http://127.0.0.1/index-redirect.html?environment=alpha&ticket=123&token=456&redirectUrl=` + encodeURIComponent('http://127.0.0.1/index-app.html'),
                },
                dest: {
                    "flag": true,
                    "msg": "",
                    "data": {
                        "environment": "alpha",
                        "ticket": "123",
                        "token": "999",
                        "appKey": "",
                        "redirectUrl": "http://127.0.0.1/index-app.html",
                        "origin_redirectUrl_decode": "http://127.0.0.1/index-app.html",
                        "origin_redirectUrl_encode": encodeURIComponent('http://127.0.0.1/index-app.html')
                    }
                }
            },

        ];

        _.each(testCaseList, function (testCase) {
            const src = testCase.src;
            const dest = testCase.dest;

            it(testCase.msg, function (done) {
                const aRedirectHelper = new NL.systemDocking.sso.frontend.redirectHelper({
                    redirectType: '110',
                    url: src.url,
                    redirectTypeParameter: {
                        '110': {
                            get_token: function (args) {
                                const ticket = args.ticket;
                                const data = {};
                                data.token = `999`;
                                return Q.resolve(data);
                            }
                        }
                    }
                });

                aRedirectHelper.init();
                aRedirectHelper.getReturnValue().then(function (aReturnValue) {
                    console.log(aReturnValue);
                    expect(aReturnValue.flag).to.eql(dest.flag);
                    expect(aReturnValue.msg).to.eql(dest.msg);

                    expect(aReturnValue.data.environment).to.eql(dest.data.environment);
                    expect(aReturnValue.data.ticket).to.eql(dest.data.ticket);
                    expect(aReturnValue.data.token).to.eql(dest.data.token);

                    const uri_expect = URI(dest.data.redirectUrl);
                    const uri_actual = URI(aReturnValue.data.redirectUrl);

                    expect(uri_actual.path()).to.eql(uri_expect.path());

                    const queryStringObject_expect = NL.uri.get_query_string_object({uri: dest.data.redirectUrl});
                    const queryStringObject_actual = NL.uri.get_query_string_object({uri: aReturnValue.data.redirectUrl});

                    expect(queryStringObject_actual.appkey).to.eql(queryStringObject_expect.appkey);
                    expect(queryStringObject_actual).to.have.property('nlVersion');

                    done();
                }).catch(function (ex) {
                    console.error(ex);
                });
            });
        })

    });

});