// //<editor-fold desc="node core modules">
// const path = require('path');
// //</editor-fold>

//<editor-fold desc="npm modules">
// const fs = require('fs-extra');
const extend = require('extend');
const Q = require('q');
const _ = require('underscore');
const moment = require('moment');
const URI = require('urijs');
//</editor-fold>

const delete_object_property = require(`../../../../common/delete_object_property`);

const get_query_string_object = require(`../../../../uri/get_query_string_object`);
const join = require(`../../../../uri/join`);

const set_environment = require(`../../../../model/set_environment`);
const set_token = require(`../../../../model/set_token`);

const srcReturnValue = {
    flag: false,
    msg: '',
    data: {
        environment: '',
        token: '',
        redirectUrl: ''
    }
};

/**
 * NL类库
 *
 * @module NL
 */

/**
 * 对接sso，前端页面重定向帮助类
 *
 * @class NL.systemDocking.sso.frontend.redirectHelper
 * @constructor
 *
 * @param {object} pParameter
 *
 * @param {string} pParameter.url 访问地址，默认为【window.location.href】
 * @param {string} pParameter.redirectType 重定向类型，默认值为000
 * <style>
 * .text-indent-2em{
 *     text-indent: 2em;
 * }
 * .temp-row{
 * line-height: 1.5em;
 * }
 * </style>
 * <div class="temp-row">1）00X：queryString中传入ticket参数，另外必须传入appKey参数</div>
 * <div class="temp-row text-indent-2em">1.1）000：returnValue.redirectUrl=`index-{YOUR_APPKEY}.html?appkey={YOUR_APPKEY}`</div>
 * <div class="temp-row text-indent-2em">1.1）001：returnValue.redirectUrl=`../index-{YOUR_APPKEY}.html?appkey={YOUR_APPKEY}`</div>
 *
 * <div class="temp-row">2）10X：queryString中传入token参数，另外必须传入appKey参数</div>
 * <div class="temp-row text-indent-2em">2.1）100：returnValue.redirectUrl=`index-{YOUR_APPKEY}.html?appkey={YOUR_APPKEY}`</div>
 * <div class="temp-row text-indent-2em">2.2）101：returnValue.redirectUrl=`../{YOUR_APPKEY}/index-{YOUR_APP_KEY}.html?appkey={YOUR_APPKEY}`</div>
 *
 * <div class="temp-row">3）110：queryString中传入ticket参数或者token参数（优先ticket参数），另外必须传入environment和redirectUrl参数</div>
 * <div class="temp-row" style="color:red;font-weight:bold;text-decoration:line-through;">4）210：queryString中传入requestId，另外必须传入environment和redirectUrl参数（已抛弃）</div>
 *
 * <br >
 * <div class="temp-row">redirectType值的定义原则</div>
 * <div class="temp-row">redirectType={第一位}{第二位}{第三位}</div>
 * <div class="temp-row">第一位：登录授权主要参数不同，例如000和100，0：queryString中传入ticket参数，1：queryString中传入token参数</div>
 * <div class="temp-row">第二位：登录授权次要参数不同，例如100和110，0：queryString中无须传入次要参数，1：queryString中传入appKey参数</div>
 * <div class="temp-row">第三位：返回值某些数据值不同，例如100和101，</div>
 * <div class="temp-row text-indent-2em">0：returnValue.redirectUrl=`index-{YOUR_APPKEY}.html?appkey={YOUR_APPKEY}`</div>
 * <div class="temp-row text-indent-2em">1：returnValue.redirectUrl=`../{YOUR_APPKEY}/index-{YOUR_APP_KEY}.html?appkey={YOUR_APPKEY}`</div>
 *
 * @param {object} pParameter.redirectTypeParameter 特定redirectType要传入的参数
 *
 * @param {object} pParameter.redirectTypeParameter['000'] redirectType=000时要传入的参数
 * @param {function} pParameter.redirectTypeParameter['000'].getToken 获取token
 * <br>1）方法签名：function(pParameter){}
 * <br>2）pParameter格式为:{ticket:'xxx'}
 * <br>3）返回值必须为一延迟对象，并且该延迟对象resolve时，必须传入【{token:'xxx'}】这样格式的对象
 *
 * @param {object} pParameter.redirectTypeParameter['001'] redirectType=001时要传入的参数
 * @param {function} pParameter.redirectTypeParameter['001'].getToken 获取token
 * <br>1）方法签名：function(pParameter){}
 * <br>2）pParameter格式为:{ticket:'xxx'}
 * <br>3）返回值必须为一延迟对象，并且该延迟对象resolve时，必须传入【{token:'xxx'}】这样格式的对象
 *
 * @param {object} pParameter.redirectTypeParameter['100'] redirectType=100时要传入的参数
 * 
 * @param {object} pParameter.redirectTypeParameter['110'] redirectType=110时要传入的参数
 * @param {function} pParameter.redirectTypeParameter['110'].get_token 获取token，仅传入的参数为ticket时，会调用该方法
 *
 * @param {object} pParameter.redirectTypeParameter['210'] redirectType=210时要传入的参数
 * @param {function} pParameter.redirectTypeParameter['210'].get_token 获取token
 * <br>1）方法签名：function(pParameter){}
 * <br>2）pParameter格式为:{requestId:'xxx'}
 * <br>3）返回值必须为一延迟对象，并且该延迟对象resolve时，必须传入【{token:'xxx'}】这样格式的对象
 *
 * @return {object} NL.systemDocking.sso.frontend.redirectHelper实例
 *
 * @demo src/system-docking/sso/frontend/redirectHelper/demo/ctor-redirectType-000.js
 * @demo src/system-docking/sso/frontend/redirectHelper/demo/ctor-redirectType-001.js
 * @demo src/system-docking/sso/frontend/redirectHelper/demo/ctor-redirectType-100.js
 * @demo src/system-docking/sso/frontend/redirectHelper/demo/ctor-redirectType-101.js
 * @demo src/system-docking/sso/frontend/redirectHelper/demo/ctor-redirectType-110.js
 * @demo src/system-docking/sso/frontend/redirectHelper/demo/ctor-redirectType-210.js
 *
 */
module.exports = function (pInitParameter) {
    const __this = this;

    const defaultInitParameter = {};
    defaultInitParameter.url = '';
    defaultInitParameter.redirectType = '000';
    defaultInitParameter.redirectTypeParameter = {};

    defaultInitParameter.redirectTypeParameter['000'] = {};
    defaultInitParameter.redirectTypeParameter['000'].getToken = null;

    defaultInitParameter.redirectTypeParameter['001'] = {};
    defaultInitParameter.redirectTypeParameter['001'].getToken = null;

    defaultInitParameter.redirectTypeParameter['100'] = {};

    defaultInitParameter.redirectTypeParameter['101'] = {};

    defaultInitParameter.redirectTypeParameter['110'] = {};
    defaultInitParameter.redirectTypeParameter['110'].get_token = null;

    defaultInitParameter.redirectTypeParameter['210'] = {};
    defaultInitParameter.redirectTypeParameter['210'].get_token = null;

    const initParameter = extend(true, {}, defaultInitParameter, pInitParameter);
    const redirectType = initParameter.redirectType;
    let url = initParameter.url;
    if (!url) {
        url = global.location.href;
    }

    let promiseReturnValue;

    const checkInitParameter = function () {
        const r = {};
        r.flag = true;
        r.msg = '';

        switch (redirectType) {
            case '000':
                if (!_.isFunction(initParameter.redirectTypeParameter['000'].getToken)) {
                    r.msg = `在redirectType=${redirectType}下，redirectTypeParameter["000"].getToken必须实现！`;
                    r.flag = false;
                    console.error(r.msg);
                    return r;
                }
                break;

            case '001':
                if (!_.isFunction(initParameter.redirectTypeParameter['001'].getToken)) {
                    r.msg = `在redirectType=${redirectType}下，redirectTypeParameter["001"].getToken必须实现！`;
                    r.flag = false;
                    console.error(r.msg);
                    return r;
                }
                break;

            case '100':
                break;

            case '101':
                break;

            case '110':
                break;
        }

        return r;
    };

    const returnValue_checkInitParameter = checkInitParameter();
    if (!returnValue_checkInitParameter.flag) {
        return;
    }

    const init = function () {
        const queryStringObject = get_query_string_object({
            url: url,
            isLowerCase: true
        });

        const process_redirectType_00X = function () {
            const returnValue = extend(true, {}, srcReturnValue);
            const ticket_encode = queryStringObject.ticket;
            if (!ticket_encode) {
                returnValue.msg = '获取ticket失败！';
                return Q.reject(returnValue);
            }
            const ticket = decodeURIComponent(ticket_encode);

            const appKey = queryStringObject.appkey;
            if (!appKey) {
                returnValue.msg = '获取appKey值失败！';
                return Q.reject(returnValue);
            }

            return Q.when()
                .then(function () {
                    return initParameter.redirectTypeParameter[redirectType].getToken({
                        ticket: ticket
                    });
                })
                .then(function (data) {
                    const token = data.token;
                    const url_001 = `index-${appKey}.html`;
                    const aQueryStringObject = get_query_string_object({
                        url: url
                    });
                    const bQueryStringObject = extend(true, {}, aQueryStringObject);
                    delete bQueryStringObject.redirecttype;
                    delete bQueryStringObject.ticket;
                    bQueryStringObject.nlVersion = moment().format('YYYYMMDDHHmmss');

                    let url_002;
                    switch (redirectType) {
                        case '000':
                            url_002 = URI(url_001).query(bQueryStringObject).toString();
                            break;

                        case '001':
                            url_002 = URI(url_001).query(bQueryStringObject).toString();
                            url_002 = join('../', appKey, url_002);
                            break;
                    }

                    returnValue.flag = true;
                    returnValue.data.token = token;
                    returnValue.data.appKey = appKey;
                    returnValue.data.redirectUrl = url_002;

                    return Q.when(returnValue);
                });
        };

        const process_redirectType_10X = function () {
            const returnValue = extend(true, {}, srcReturnValue);
            const token_encode = queryStringObject.token;
            if (!token_encode) {
                returnValue.msg = '获取token值失败！';
                return Q.reject(returnValue);
            }
            const token = decodeURIComponent(token_encode);

            // console.log('token=');
            // console.log(token);

            const appKey = queryStringObject.appkey;
            if (!appKey) {
                returnValue.msg = '获取appKey值失败！';
                return Q.reject(returnValue);
            }

            const url_001 = `index-${appKey}.html`;
            const aQueryStringObject = get_query_string_object({
                url: url,
            });
            let bQueryStringObject = extend(true, {}, aQueryStringObject);
            bQueryStringObject = delete_object_property({
                o: bQueryStringObject,
                name: `redirectType`
            });
            bQueryStringObject = delete_object_property({
                o: bQueryStringObject,
                name: `redirectUrl`
            });
            bQueryStringObject = delete_object_property({
                o: bQueryStringObject,
                name: `token`
            });
            bQueryStringObject.nlVersion = moment().format('YYYYMMDDHHmmss');

            let url_002;
            switch (redirectType) {
                case '100':
                    url_002 = URI(url_001).query(bQueryStringObject).toString();
                    break;

                case '101':
                    url_002 = URI(url_001).query(bQueryStringObject).toString();
                    url_002 = join('../', appKey, url_002);
                    break;
            }

            returnValue.flag = true;
            returnValue.data.token = token;
            returnValue.data.appKey = appKey;
            returnValue.data.redirectUrl = url_002;

            return Q.when(returnValue);
        };

        const process_redirectType_110 = function () {
            const returnValue = extend(true, {}, srcReturnValue);

            const environment = queryStringObject.environment;
            if (!environment) {
                returnValue.msg = 'process_redirectType_110 parameter error, code=001';
                return Q.reject(returnValue);
            }

            const get_auth_type = function () {
                let r = ``;

                const ticket = queryStringObject.ticket;
                if (ticket) {
                    r = `ticket`;
                    return r;
                }

                const token = queryStringObject.token;
                if (token) {
                    r = `token`;
                    return r;
                }

                return r;
            };
            const auth_type = get_auth_type();

            if (!auth_type) {
                returnValue.msg = 'process_redirectType_110 parameter error, code=002';
                return Q.reject(returnValue);
            }

            const get_auth_parameter = function () {

                const process_auth_type_ticket = function () {

                    const __ticket = queryStringObject.ticket;
                    const ticket = decodeURIComponent(__ticket);

                    const get_token = initParameter.redirectTypeParameter['110'].get_token;
                    if (!_.isFunction(get_token)) {
                        const msg = 'process_auth_type_ticket parameter error, code=001';
                        console.error(msg);
                        return Q.reject({msg: msg});
                    }

                    return Q.when()
                        .then(function () {
                            return get_token({
                                ticket: ticket,
                                __ticket: __ticket,
                            });
                        })
                        .then(function (data) {
                            const token = data.token;

                            const d = {};
                            d.ticket = ticket;
                            d.token = token;

                            return Q.when(d);
                        })
                        ;

                };

                const process_auth_type_token = function () {
                    const __token = queryStringObject.token;
                    const token = decodeURIComponent(__token);

                    const d = {};
                    d.ticket = ``;
                    d.token = token;

                    return Q.when(d);
                };

                let r;
                switch (auth_type) {
                    case `ticket`:
                        r = process_auth_type_ticket();
                        break;

                    case `token`:
                        r = process_auth_type_token();
                        break;
                }
                return r;
            };

            return Q.when()
                .then(function () {
                    return get_auth_parameter();
                })
                .then(function (data) {
                    const ticket = data.ticket;
                    const token = data.token;

                    const redirectUrl_encode = queryStringObject.redirecturl;
                    if (!redirectUrl_encode) {
                        returnValue.msg = 'process_redirectType_110 parameter error, code=003';
                        return Q.reject(returnValue);
                    }
                    const redirectUrl_001 = decodeURIComponent(redirectUrl_encode);

                    const aQueryStringObject = get_query_string_object({
                        url: url,
                    });

                    let bQueryStringObject = extend(true, {}, aQueryStringObject);
                    bQueryStringObject = delete_object_property({
                        o: bQueryStringObject,
                        name: 'redirectType'
                    });
                    bQueryStringObject = delete_object_property({
                        o: bQueryStringObject,
                        name: 'token'
                    });
                    bQueryStringObject = delete_object_property({
                        o: bQueryStringObject,
                        name: 'redirectUrl'
                    });
                    bQueryStringObject = delete_object_property({
                        o: bQueryStringObject,
                        name: 'environment'
                    });
                    bQueryStringObject.nlVersion = moment().format('YYYYMMDDHHmmss');

                    const redirectUrl_002 = URI(redirectUrl_001).query(bQueryStringObject).toString();

                    returnValue.flag = true;
                    returnValue.data.environment = environment;
                    returnValue.data.ticket = ticket;
                    returnValue.data.token = token;
                    returnValue.data.appKey = '';
                    returnValue.data.redirectUrl = redirectUrl_002;
                    returnValue.data.origin_redirectUrl_encode = redirectUrl_encode;
                    returnValue.data.origin_redirectUrl_decode = redirectUrl_001;

                    return Q.when(returnValue);
                })
                .catch(function (ex) {
                    console.error(ex);
                    if (ex) {
                        if (ex.stack) {
                            console.error(ex.stack);
                        }
                    }
                    return Q.reject(ex);
                })
                ;


        };

        const process_redirectType_210 = function () {
            const returnValue = extend(true, {}, srcReturnValue);

            const environment = queryStringObject.environment;
            if (!environment) {
                returnValue.msg = 'process_redirectType_210 parameter error, code=001';
                return Q.reject(returnValue);
            }

            const requestId = queryStringObject.requestid;
            if (!requestId) {
                returnValue.msg = 'process_redirectType_210 parameter error, code=002';
                return Q.reject(returnValue);
            }

            const get_token = initParameter.redirectTypeParameter[`210`].get_token;
            if (!_.isFunction(get_token)) {
                returnValue.msg = 'process_redirectType_210 parameter error, code=003';
                return Q.reject(returnValue);
            }

            return Q.when()
                .then(function () {
                    return get_token({requestId: requestId});
                })
                .then(function (data) {
                    if (!data) {
                        returnValue.msg = 'process_redirectType_210 parameter error, code=004';
                        return Q.reject(returnValue);
                    }
                    const token = data.token;

                    // console.log('token=');
                    // console.log(token);

                    const redirectUrl_encode = queryStringObject.redirecturl;
                    if (!redirectUrl_encode) {
                        returnValue.msg = 'process_redirectType_210 parameter error, code=005';
                        return Q.reject(returnValue);
                    }
                    const redirectUrl_001 = decodeURIComponent(redirectUrl_encode);

                    const aQueryStringObject = get_query_string_object({
                        url: url,
                    });

                    let bQueryStringObject = extend(true, {}, aQueryStringObject);
                    bQueryStringObject = delete_object_property({
                        o: bQueryStringObject,
                        name: 'redirectType'
                    });
                    bQueryStringObject = delete_object_property({
                        o: bQueryStringObject,
                        name: 'token'
                    });
                    bQueryStringObject = delete_object_property({
                        o: bQueryStringObject,
                        name: 'redirectUrl'
                    });
                    bQueryStringObject = delete_object_property({
                        o: bQueryStringObject,
                        name: 'environment'
                    });
                    bQueryStringObject = delete_object_property({
                        o: bQueryStringObject,
                        name: 'requestId'
                    });
                    bQueryStringObject.nlVersion = moment().format('YYYYMMDDHHmmss');

                    const redirectUrl_002 = URI(redirectUrl_001).query(bQueryStringObject).toString();

                    returnValue.flag = true;
                    returnValue.data.environment = environment;
                    returnValue.data.requestId = requestId;
                    returnValue.data.token = token;
                    returnValue.data.redirectUrl = redirectUrl_002;
                    returnValue.data.origin_redirectUrl_encode = redirectUrl_encode;
                    returnValue.data.origin_redirectUrl_decode = redirectUrl_001;

                    return Q.when(returnValue);
                })
                .catch(function (ex) {
                    console.error(ex);
                    if (ex) {
                        if (ex.stack) {
                            console.error(ex.stack);
                        }
                    }
                    return Q.reject(ex);
                });


        };

        switch (redirectType) {
            case '000':
            case '001':
                promiseReturnValue = process_redirectType_00X();
                break;

            case '100':
            case '101':
                promiseReturnValue = process_redirectType_10X();
                break;

            case '110':
                promiseReturnValue = process_redirectType_110();
                break;

            case '210':
                promiseReturnValue = process_redirectType_210();
                break;
        }

        promiseReturnValue.then(
            function (returnValue) {
                if (!returnValue.flag) {
                    console.error(returnValue.msg);
                    return;
                }

                switch (redirectType) {
                    case '000':
                    case '001':
                    case '100':
                    case '101':
                        set_token({
                            token: returnValue.data.token
                        });
                        break;

                    case '110':
                    case '210':
                        set_token({
                            token: returnValue.data.token
                        });
                        set_environment({
                            environment: returnValue.data.environment
                        });
                        break;
                }

            },
            function (data) {
                console.error(data);
            }
        );
    };

    const getReturnValue = function () {
        return promiseReturnValue;
    };

    /**
     * 初始化
     * @method init
     */
    __this.init = init;
    /**
     * 获取返回值
     * @method getReturnValue
     *
     * @return {object} promise
     * <br>promise.then(function(data){})
     * <br>data的格式为：{flag:false,msg:'',data:{environment: '', token:'', appKey:'', redirectUrl:''}}
     */
    __this.getReturnValue = getReturnValue;

};