const path = require(`path`);
const fs = require(`fs-extra`);

const extend = require(`extend`);
const Q = require(`q`);
const _ = require(`underscore`);
const moment = require(`moment`);
const URI = require(`urijs`);
const axios = require(`axios`);

//https://developers.weixin.qq.com/doc/offiaccount/Basic_Information/Get_access_token.html

const get = function (pParameter) {
    if (!pParameter) pParameter = {};
    const grant_type = pParameter.grant_type || `client_credential`;
    const appid = pParameter.appid;
    const secret = pParameter.secret;

    const url = `https://api.weixin.qq.com/cgi-bin/token`;

    return Q.when()
        .then(function () {
            return axios({
                method: `get`,
                url,
                params: {
                    grant_type,
                    appid,
                    secret,
                },

                // proxy: {
                //     host: '127.0.0.1',
                //     port: 8888,
                // },
            });
        })
        .then(function (response) {
            const data = response.data;
            // console.log(JSON.stringify(data, null, 4));

            const access_token = data.access_token;
            if (!access_token) {
                const msg = `backend/lib/middlewares/api/weixin/cgi-bin/token parameter error, code=001`;
                console.error(msg);
                console.trace();
                return Q.reject({msg: msg});
            }
            return Q.resolve(data);
        })
        .catch(function (ex) {
            if (ex) {
                const enable_log_error = !ex.disable_log_error;
                if (enable_log_error) {
                    console.error(ex);
                    console.trace();
                    if (ex.stack) {
                        console.error(ex.stack);
                    }
                }
            }
            return Q.reject(ex);
        })
        ;
};

module.exports = get;
