const path = require(`path`);
const fs = require(`fs-extra`);
const extend = require(`extend`);
const Q = require(`q`);
// const _ = require(`underscore`);
// const moment = require(`moment`);
// const axios = require(`axios`);
const WxPay = require(`wechatpay-node-v3`);

//https://pay.weixin.qq.com/wiki/doc/apiv3/apis/chapter3_1_1.shtml
const transactions_jsapi = function (pParameter) {
    if (!pParameter) pParameter = {};

    const defaultParameter = {};
    defaultParameter.appid = ``;//应用ID
    defaultParameter.mchid = ``;//直连商户号
    defaultParameter.path_file_publicKey = ``;//商户号证书公钥文件路径
    defaultParameter.path_file_privateKey = ``;//商户号证书私钥文件路径
    defaultParameter.description = ``;//商品描述
    defaultParameter.out_trade_no = ``;//商户订单号
    defaultParameter.notify_url = ``;//通知地址
    defaultParameter.amount = {
        total: 1,
    };
    defaultParameter.payer = {
        openid: ``,
    };

    const parameter = extend(true, {}, defaultParameter, pParameter);

    const appid = parameter.appid;
    const mchid = parameter.mchid;
    const path_file_publicKey = parameter.path_file_publicKey;
    const path_file_privateKey = parameter.path_file_privateKey;
    const description = parameter.description;
    const out_trade_no = parameter.out_trade_no;
    const notify_url = parameter.notify_url;
    const total = parameter.amount.total;
    const openid = parameter.payer.openid;

    if (!appid) {
        const msg = `transactions_jsapi parameter error, code=001`;
        console.error(msg);
        console.trace();
        return Q.reject({msg: msg});
    }
    if (!mchid) {
        const msg = `transactions_jsapi parameter error, code=002`;
        console.error(msg);
        console.trace();
        return Q.reject({msg: msg});
    }
    if (!path_file_publicKey) {
        const msg = `transactions_jsapi parameter error, code=003`;
        console.error(msg);
        console.trace();
        return Q.reject({msg: msg});
    }
    if (!fs.existsSync(path_file_publicKey)) {
        const msg = `transactions_jsapi parameter error, code=004`;
        console.error(msg);
        console.trace();
        return Q.reject({msg: msg});
    }
    if (!path_file_privateKey) {
        const msg = `transactions_jsapi parameter error, code=005`;
        console.error(msg);
        console.trace();
        return Q.reject({msg: msg});
    }
    if (!fs.existsSync(path_file_privateKey)) {
        const msg = `transactions_jsapi parameter error, code=006`;
        console.error(msg);
        console.trace();
        return Q.reject({msg: msg});
    }
    if (!description) {
        const msg = `transactions_jsapi parameter error, code=007`;
        console.error(msg);
        console.trace();
        return Q.reject({msg: msg});
    }
    if (!out_trade_no) {
        const msg = `transactions_jsapi parameter error, code=008`;
        console.error(msg);
        console.trace();
        return Q.reject({msg: msg});
    }
    if (!notify_url) {
        const msg = `transactions_jsapi parameter error, code=009`;
        console.error(msg);
        console.trace();
        return Q.reject({msg: msg});
    }
    if (total <= 0) {
        const msg = `transactions_jsapi parameter error, code=010`;
        console.error(msg);
        console.trace();
        return Q.reject({msg: msg});
    }
    if (!openid) {
        const msg = `transactions_jsapi parameter error, code=011`;
        console.error(msg);
        console.trace();
        return Q.reject({msg: msg});
    }

    const pay = new WxPay({
        appid,
        mchid,
        publicKey: fs.readFileSync(path_file_publicKey), // 公钥
        privateKey: fs.readFileSync(path_file_privateKey), // 秘钥
    });

    const params = {
        description,
        out_trade_no,
        notify_url,
        amount: {
            total,
        },
        payer: {
            openid,
        },
    };

    return Q.when()
        .then(function () {
            return pay.transactions_jsapi(params);
        })
        .catch(function (ex) {
            if (ex) {
                const enable_log_error = !ex.disable_log_error;
                if (enable_log_error) {
                    console.error(ex);
                    console.trace();
                    if (ex.stack) {
                        console.error(ex.stack);
                    }
                }
            }
            return Q.reject(ex);
        })
        ;
};

module.exports = transactions_jsapi;
